import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class BlurFilter extends EffectFilter
{	
	private transient JSlider blurSlider;
	private transient FloatImage tempBuffer;
	
	private int blurLevel;

	public BlurFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Blur Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		blurLevel = 2;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		blurLevel = blurSlider.getValue();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		tempBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		blurSlider = new JSlider(JSlider.HORIZONTAL);
		blurSlider.setMajorTickSpacing(1);
		blurSlider.setPaintTicks(true);
		blurSlider.setMinimum(1);
		blurSlider.setMaximum(6);
		blurSlider.setBorder(BorderFactory.createTitledBorder("Blur Level"));
		blurSlider.setValue(blurLevel);
		
		guiWindow.add(blurSlider);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "Input";
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public void doFasterBlur(int blurLevel, FloatImage temp)
	{
		int blurSide = (2 * blurLevel) + 1;
		
		double divisor = (double)(blurSide * blurSide);
		
		if(temp != null)
		{
			if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
			{
				internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());
				tempBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
			}
			
			int imgWidth = internalBuffer.getWidth();
			int imgHeight = internalBuffer.getHeight();
			
			double[][] ored = tempBuffer.getChannelData(0);
			double[][] ogreen = tempBuffer.getChannelData(1);
			double[][] oblue = tempBuffer.getChannelData(2);
			
			double[][] ired = temp.getChannelData(0);
			double[][] igreen = temp.getChannelData(1);
			double[][] iblue = temp.getChannelData(2);
			
			double sumr, sumg, sumb;
			
			
			int i, j;
			
			// first we add horizontally
			for(j = 0; j < imgHeight; ++j)
			{
				// we have to manually add up the first blurSide pixels
				sumr = 0.0;
				sumb = 0.0;
				sumg = 0.0;
				
				
				for(i = 0; i < blurSide; ++i)
				{
					sumr += ired[i][j];
					sumg += igreen[i][j];
					sumb += iblue[i][j];	
				}
				
				ored[blurLevel][j] = sumr;
				ogreen[blurLevel][j] = sumg;
				oblue[blurLevel][j] = sumb;
				
				for(i = blurLevel+1; i < imgWidth - blurLevel; ++i)
				{
					sumr = sumr - ired[i - blurLevel - 1][j] + ired[i + blurLevel][j];
					sumg = sumg - igreen[i - blurLevel - 1][j] + igreen[i + blurLevel][j];
					sumb = sumb - iblue[i - blurLevel - 1][j] + iblue[i + blurLevel][j];
					
					ored[i][j] = sumr;
					ogreen[i][j] = sumg;
					oblue[i][j] = sumb; 
				}
			}

			// we are now reading from the temp buffer
			ired = tempBuffer.getChannelData(0);
			igreen = tempBuffer.getChannelData(1);
			iblue = tempBuffer.getChannelData(2);
			
			// and writing to the actual internalBuffer
			ored = internalBuffer.getChannelData(0);
			ogreen = internalBuffer.getChannelData(1);
			oblue = internalBuffer.getChannelData(2);
			
			// now we average vertically
			for(i = 0; i < imgWidth; ++i)
			{
				// we have to manually add up the first blurSide pixels
				sumr = 0.0;
				sumb = 0.0;
				sumg = 0.0;
				
				
				for(j = 0; j < blurSide; ++j)
				{
					sumr += ired[i][j];
					sumg += igreen[i][j];
					sumb += iblue[i][j];	
				}
				
				ored[i][blurLevel] = sumr;
				ogreen[i][blurLevel] = sumg;
				oblue[i][blurLevel] = sumb;
				
				for(j = blurLevel+1; j < imgHeight - blurLevel; ++j)
				{
					sumr = sumr - ired[i][j - blurLevel - 1] + ired[i][j + blurLevel];
					sumg = sumg - igreen[i][j - blurLevel - 1] + igreen[i][j + blurLevel];
					sumb = sumb - iblue[i][j - blurLevel - 1] + iblue[i][j + blurLevel];
					
					ored[i][j] = sumr / divisor;
					ogreen[i][j] = sumg / divisor;
					oblue[i][j] = sumb / divisor; 
				}
			}
			
			// we still have to deal with the edges: we'll just leave them
			// unblurred for now; we have to switch the input back to the
			// unblurred image
			
			/*
			ired = temp.getChannelData(0);
			igreen = temp.getChannelData(1);
			iblue = temp.getChannelData(2);
			*/
			
			// this handles the top and bottom edges
			for(i = 0; i < imgWidth; ++i)
			{
				for(j = 0; j < blurLevel+1; ++j)
				{
					ored[i][j] = -1.0;
					ogreen[i][j] = -1.0;
					oblue[i][j] = -1.0;
					
					ored[i][j+imgHeight-blurLevel-1] = -1.0;
					ogreen[i][j+imgHeight-blurLevel-1] = -1.0;
					oblue[i][j+imgHeight-blurLevel-1] = -1.0;
				}
			}
			
			// this handles the left and right edges
			for(j = 0; j < imgHeight; ++j)
			{
				for(i = 0; i < blurLevel; ++i)
				{
					ored[i][j] = -1.0;
					ogreen[i][j] = -1.0;
					oblue[i][j] = -1.0;
					
					ored[i+imgWidth-blurLevel][j] = -1.0;
					ogreen[i+imgWidth-blurLevel][j] = -1.0;
					oblue[i+imgWidth-blurLevel][j] = -1.0;
				}
			}
			
		}
	}
	
	public void doNaiveBlur(int blurLevel, FloatImage temp)
	{
		int blurSide = (2 * blurLevel) + 1;
		
		double divisor = (double)(blurSide * blurSide);
		
		if(temp != null)
		{
			if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
			{
				internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());
				tempBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
			}
			
			int imgWidth = internalBuffer.getWidth();
			int imgHeight = internalBuffer.getHeight();
			
			double[][] ored = internalBuffer.getChannelData(0);
			double[][] ogreen = internalBuffer.getChannelData(1);
			double[][] oblue = internalBuffer.getChannelData(2);
			
			double[][] ired = temp.getChannelData(0);
			double[][] igreen = temp.getChannelData(1);
			double[][] iblue = temp.getChannelData(2);
			
			double sumr, sumg, sumb;
			
			
			// this blurs the body of the image
			for(int i = blurLevel; i < imgWidth - blurLevel; ++i)
			{
				for(int j = blurLevel; j < imgHeight - blurLevel; ++j)
				{
					sumr = 0.0;
					sumg = 0.0;
					sumb = 0.0;
					
					for(int x = i - blurLevel; x <= i + blurLevel; ++x)
					{
						for(int y = j - blurLevel; y <= j + blurLevel; ++y)
						{
							sumr += ired[x][y];
							sumg += igreen[x][y];
							sumb += iblue[x][y]; 
						}
					}
					
					ored[i][j] = sumr / divisor;
					ogreen[i][j] = sumg / divisor;
					oblue[i][j] = sumb / divisor;
				}
			}
			
			// we still have to deal with the edges: we'll just leave them
			// unblurred for now
			
			// this handles the top and bottom edges
			for(int i = 0; i < imgWidth; ++i)
			{
				for(int j = 0; j < blurLevel; ++j)
				{
					ored[i][j] = ired[i][j];
					ogreen[i][j] = igreen[i][j];
					oblue[i][j] = iblue[i][j];
					
					ored[i][j+imgHeight-blurLevel] = ired[i][j+imgHeight-blurLevel];
					ogreen[i][j+imgHeight-blurLevel] = igreen[i][j+imgHeight-blurLevel];
					oblue[i][j+imgHeight-blurLevel] = iblue[i][j+imgHeight-blurLevel];
				}
			}
			
			// this handles the left and right edges
			for(int j = 0; j < imgHeight; ++j)
			{
				for(int i = 0; i < blurLevel; ++i)
				{
					ored[i][j] = ired[i][j];
					ogreen[i][j] = igreen[i][j];
					oblue[i][j] = iblue[i][j];
					
					ored[i+imgWidth-blurLevel][j] = ired[i+imgWidth-blurLevel][j];
					ogreen[i+imgWidth-blurLevel][j] = igreen[i+imgWidth-blurLevel][j];
					oblue[i+imgWidth-blurLevel][j] = iblue[i+imgWidth-blurLevel][j];
				}
			}
		}
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		blurLevel = blurSlider.getValue();
		int blurSide = (2 * blurLevel) + 1;
		
		double divisor = (double)(blurSide * blurSide);
		
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			//doNaiveBlur(blurLevel, temp);
			
			// Hopefully this should be significantly faster
			doFasterBlur(blurLevel, temp);
			
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}