import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class CompositeFilter extends EffectFilter
{	
	public CompositeFilter()
	{
		filterInputs = new FilterLink[3];
		filterInputs[0] = null;
		filterInputs[1] = null;
		filterInputs[2] = null;
		dead = false;
		filterName = "Composite Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		guiWindow.add(new JLabel("Compositer"));
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		switch(inputNum)
		{
		case 0:
			return "Input A";
		case 1:
			return "Input B";
		case 2:
			return "Mask Input";
		default:
			return "Unknown Input";
		}
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null && filterInputs[1] != null && filterInputs[2] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			FloatImage temp2 = filterInputs[1].getOutput();
			FloatImage mask = filterInputs[2].getOutput();
			
			if(temp != null && temp2 != null)
			{
				int imgWidth = Math.min(mask.getWidth(), Math.min(temp.getWidth(), temp2.getWidth()));
				int imgHeight = Math.min(mask.getHeight(), Math.min(temp.getHeight(), temp2.getHeight()));
					
				if(internalBuffer.getWidth() != imgWidth || 
				 internalBuffer.getHeight() != imgHeight)
				{
					internalBuffer = new FloatImage(3, imgWidth, imgHeight);	
				}
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				double[][] ired2 = temp2.getChannelData(0);
				double[][] igreen2 = temp2.getChannelData(1);
				double[][] iblue2 = temp2.getChannelData(2);
				
				double[][] mred = mask.getChannelData(0);
				
				double amt, ramt;
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
							amt = (mred[i][j] + 1.0) / 2.0;
							ramt = 1.0 - amt;
						
							ored[i][j] = (ired[i][j] * amt) + (ired2[i][j] * ramt);
							ogreen[i][j] = (igreen[i][j] * amt) + (igreen2[i][j] * ramt);
							oblue[i][j] = (iblue[i][j] * amt) + (iblue2[i][j] * ramt);
					}
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}