import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class DelayFilter extends EffectFilter
{	
	private transient JSlider delaySlider;
	private int delayAmount;
	private FloatImage[] dbuffer;
	private int inHead;
	private int outHead;

	public DelayFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Delay Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		dbuffer = new FloatImage[15];
		for(int i = 0; i < dbuffer.length; ++i)
		{
			dbuffer[i] = new FloatImage();
		}
		delayAmount = 6;
		inHead = 0;
		outHead = (inHead + delayAmount) % dbuffer.length;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		delayAmount = delaySlider.getValue();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		delaySlider = new JSlider(JSlider.HORIZONTAL);
		delaySlider.setMajorTickSpacing(2);
		delaySlider.setMinorTickSpacing(1);
		delaySlider.setMaximum(14);
		delaySlider.setMinimum(1);
		delaySlider.setPaintTicks(true);
		delaySlider.setBorder(BorderFactory.createTitledBorder("Delay Amount"));
		delaySlider.setValue(delayAmount);
		
		guiWindow.add(delaySlider);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "Input";
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public void update(long dt)
	{
		outHead = (outHead + 1) % dbuffer.length;
		inHead = (outHead + delaySlider.getValue()) % dbuffer.length;
		
		//System.out.println("U: o: " + outHead + " i: " + inHead);
		//System.out.println("----v");
		
		if(filterInputs[0] == null)
			return;
			
		FloatImage temp = filterInputs[0].getOutput();
		
		if(temp == null)
			return;
			
		int imgWidth = temp.getWidth();
		int imgHeight = temp.getHeight();
			
		if(dbuffer[inHead].getWidth() != imgWidth || dbuffer[inHead].getHeight() != imgHeight)
		{
			dbuffer[inHead] = new FloatImage(3, imgWidth, imgHeight);
		}
		
		double[][] ired = temp.getChannelData(0);
		double[][] igreen = temp.getChannelData(1);
		double[][] iblue = temp.getChannelData(2);
		
		double[][] ored = dbuffer[inHead].getChannelData(0);
		double[][] ogreen = dbuffer[inHead].getChannelData(1);
		double[][] oblue = dbuffer[inHead].getChannelData(2);
		
		for(int i = 0; i < imgWidth; ++i)
		{
			for(int j = 0; j < imgHeight; ++j)
			{
				ored[i][j] = ired[i][j];
				ogreen[i][j] = igreen[i][j];
				oblue[i][j] = iblue[i][j];
			}
		}
		
		//System.out.println("----^");
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		//System.out.println("GO: o: " + outHead);
		return dbuffer[outHead];
	}	
}