import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class DistortionFilter extends EffectFilter
{	
	private transient JSlider distortionSlider;
	private double distortionLevel;

	public DistortionFilter()
	{
		filterInputs = new FilterLink[3];
		filterInputs[0] = null;
		filterInputs[1] = null;
		filterInputs[2] = null;
		dead = false;
		filterName = "Distortion Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		distortionLevel = 5.0;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		distortionLevel = ((double)distortionSlider.getValue()) / 10.0;
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		distortionSlider = new JSlider(JSlider.HORIZONTAL);
		distortionSlider.setMajorTickSpacing(10);
		distortionSlider.setMinorTickSpacing(5);
		distortionSlider.setMinimum(-500);
		distortionSlider.setMaximum(500);
		distortionSlider.setPaintTicks(true);
		distortionSlider.setBorder(BorderFactory.createTitledBorder("Distortion Level"));
		distortionSlider.setValue((int)(distortionLevel*10.0));
		
		guiWindow.add(distortionSlider);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		switch(inputNum)
		{
		case 0:
			return "ImageIn";
		case 1:
			return "DistortionXIn";
		case 2:
			return "DistortionYIn";
		default:
			return "UnknownInput";
		}
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		distortionLevel = ((double)distortionSlider.getValue()) / 10.0;
		
		if(needsProcessing && filterInputs[0] != null && filterInputs[1] != null && filterInputs[2] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			FloatImage distortx = filterInputs[1].getOutput();
			FloatImage distorty = filterInputs[2].getOutput();
			
			if(temp != null && distortx != null && distorty != null)
			{
	
				int imgWidth = Math.min(distortx.getWidth(), Math.min(distorty.getWidth(), temp.getWidth()));
				int imgHeight = Math.min(distortx.getHeight(), Math.min(distorty.getHeight(), temp.getHeight()));
				
				if(imgWidth != internalBuffer.getWidth() || imgHeight != internalBuffer.getHeight())
				{
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
				}
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				double[][] dxred = distortx.getChannelData(0);
				double[][] dxgreen = distortx.getChannelData(1);
				double[][] dxblue = distortx.getChannelData(2);
				
				double[][] dyred = distorty.getChannelData(0);
				double[][] dygreen = distorty.getChannelData(1);
				double[][] dyblue = distorty.getChannelData(2);
				
				int nxg, nyg, nxb, nyb, nxr, nyr;
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						nxr = i + (int)(dxred[i][j] * distortionLevel);
						nxg = i + (int)(dxgreen[i][j] * distortionLevel);
						nxb = i + (int)(dxblue[i][j] * distortionLevel);
						
						nyr = j + (int)(dyred[i][j] * distortionLevel);
						nyg = j + (int)(dygreen[i][j] * distortionLevel);
						nyb = j + (int)(dyblue[i][j] * distortionLevel);
						
						if(nxr >= imgWidth || nxr < 0)
							nxr = 0;
							
						if(nxg >= imgWidth || nxg < 0)
							nxg = 0;
							
						if(nxb >= imgWidth || nxb < 0)
							nxb = 0;
							
						if(nyr >= imgHeight || nyr < 0)
							nyr = 0;
							
						if(nyg >= imgHeight || nyg < 0)
							nyg = 0;
							
						if(nyb >= imgHeight || nyb < 0)
							nyb = 0;
							
						ored[i][j] = ired[nxr][nyr];
						ogreen[i][j] = igreen[nxg][nyg];
						oblue[i][j] = iblue[nxb][nyb];
					}
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}