import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ArrayList;

public class DotLocatorFilter extends EffectFilter
{	
	public DotLocatorFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Dot Locator " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage(3, 2, 1);
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		guiWindow.add(new JLabel("Dot Locator"));
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "SegmentedColorIn";
	}
	
	public String getOutputName(int outputNum)
	{
		return "DATA: Pixel Coordinate Out";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null)
		{	
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null)
			{
				// First, identify all contours
				OUtil.EdgeField efield = new OUtil.EdgeField();
				efield.makeEdgeField(temp);
				
				ArrayList<OUtil.Contour> contours = efield.getOutlines();
				
				if(contours.size() == 0)
				{
					double[][] errorOut = internalBuffer.getChannelData(0);
					errorOut[0][0] = -1000.0;
					errorOut[1][0] = -1000.0;
					return internalBuffer;
				}
				
				// find largest contour...
				OUtil.Contour curContour = null;
				OUtil.Contour bestContour = null;
				
				for(int i = 0; i < contours.size(); ++i)
				{
					curContour = contours.get(i);
					if(bestContour == null || curContour.path.size() > bestContour.path.size())
					{
						bestContour = curContour;
					}
				}
				
				// calculate the CM of the best contour
				double cmx = 0.0;
				double cmy = 0.0;
				
				OUtil.Point curPoint = null;
				for(int i = 0; i < bestContour.path.size(); ++i)
				{
					curPoint = bestContour.path.get(i);
					
					cmx += curPoint.x;
					cmy += curPoint.y;
				}
				
				cmx = cmx / bestContour.path.size();
				cmy = cmy / bestContour.path.size();
				
				// stick the CM into the internal buffer
				double[][] dout = internalBuffer.getChannelData(0);
				
				dout[0][0] = cmx;
				dout[1][0] = cmy;
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}