import java.util.*;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.geom.*;
import javax.swing.*;

import javax.swing.JFrame;

public class Draw implements Runnable {
	private RoombaControl control = RoombaControl.getInstance();
	
	private Particle robot;				// location & orientation of the roomba
	private double speed = 1;			// global speed factor in cm/s
	private double scale;				// width/height of playing field
	private Vector<Point2D.Double> penPoints = new Vector<Point2D.Double>();
	private Vector<Point2D.Double> robotPoints = new Vector<Point2D.Double>();
	private Drawing dr = new Drawing("box.rob");
	
	private JFrame frame;
	private boolean modified = false;	// true when the image has been modified
	
	private double penOffsetX = 50;
	private double penOffsetY = 20;
	
	public void run() {
		PlayingField.setBoundary(0, 0, 800, 480);
		scale = PlayingField.getSquareSize();
		
		double dTotal = control.distanceMoved();
		double thetaTotal = control.angleRotatedInRadians();
		
		robot = new Particle(penOffsetX,penOffsetY,0);
		
//		Vector<DrawPoint> points = new Vector<DrawPoint>();
//		points.add(new DrawPoint(10,scale,1));
//		points.add(new DrawPoint(scale,scale,1));
//		points.add(new DrawPoint(scale,10,1));
//		points.add(new DrawPoint(10,0,1));
		
		// Set up jframe for displaying our location
	    frame = new JFrame("Particles");
	    frame.getContentPane().add(new Image());
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    frame.setSize((int)PlayingField.getBoundary().width+15,(int)PlayingField.getBoundary().height+30);
	    frame.setVisible(true);  
	    
	    // create a thread to update our image at a constant rate
	    Thread t = new Thread(new updater());
	    t.start();
	    
//	    for(DrawPoint p : points) {
	    while(dr.hasNextPoint()) {
	    	// get the next point to be drawn and make a copy of it
	    	DrawPoint p = (DrawPoint)dr.getNextPoint().clone();
	    	p.x *= scale;
	    	p.y *= scale;
	    	
	    	double cosTh = Math.cos(robot.theta);
	    	double sinTh = Math.sin(robot.theta);
	    	
	    	// find the position of the pen relative to the robot
			double penRelX = cosTh*penOffsetX + sinTh*penOffsetY;
			double penRelY = sinTh*penOffsetX - cosTh*penOffsetY;
			Point2D.Double dest = new Point2D.Double(p.x+penRelX, p.y+penRelY);

			while(robot.distance(dest) > .5) {
		    	cosTh = Math.cos(robot.theta);
		    	sinTh = Math.sin(robot.theta);
		    	
		    	// find the position of the pen relative to the robot
				penRelX = cosTh*penOffsetX + sinTh*penOffsetY;
				penRelY = sinTh*penOffsetX - cosTh*penOffsetY;
				dest = new Point2D.Double(p.x+penRelX, p.y+penRelY);
				
				
				double dX = (dest.x - robot.x)/robot.distance(dest);
				double dY = (dest.y - robot.y)/robot.distance(dest);
				double dForwards = cosTh*dX + sinTh*dY;
				double dSideways = sinTh*dX - cosTh*dY;
				
				// tell the robot to move at the given speeds
				setVelocity(dForwards,dSideways/penOffsetX);
				
				try {
					Thread.sleep(100);
				} catch(Exception e) {System.err.println(e);}
				
				// apply the motion we made to the robot
				double dNew = control.distanceMoved();
				double thetaNew = control.angleRotatedInRadians();
				double dMoved = dNew - dTotal;
				double thetaMoved = thetaNew - thetaTotal;
				
				MotionModel m = new MotionModel(dMoved, thetaMoved);
				m.exactMotion(robot);
				
				dTotal = dNew;
				thetaTotal = thetaNew;
				
				//System.out.println("robot at "+robot);
				//System.out.println("pen at "+new Point2D.Double(robot.x-penRelX,robot.y-penRelY) );
				
//				MotionModel m = new MotionModel(dForwards*speed,dSideways*speed/penOffsetX);
//				m.exactMotion(robot);

				// add the robot's current position to our picture
				try {
					synchronized(penPoints){
						//if(p.color!=0)
							penPoints.add( new Point2D.Double(robot.x-penRelX,robot.y-penRelY) );
						robotPoints.add( (Point2D.Double)robot.clone() );
						modified = true;
					}
				} catch(Exception e) {System.err.println(e);}
			}
		}
	}
	
	/*
	 * set robot velocity (slightly hackish...)
	 */
	private void setVelocity(double forward, double turn) {
        double leftWheel = forward - turn;
        double rightWheel = forward + turn;
        double normalize = Math.max(Math.abs(leftWheel), Math.abs(rightWheel));
        control.directDrive(rightWheel * speed / normalize, leftWheel * speed / normalize);
	}
	
	/*
	 * Set the absolute position of the robot
	 */
	public synchronized void setPosition(double x, double y, double theta) {
		robot = new Particle(x,y,theta);
	}
	
	/*
	 * updater is responsible for periodically repainting the image
	 */
	class updater implements Runnable {
		public void run() {
			while(true) {
				if( modified == true ) {
					frame.repaint();
					modified = false;
				}
				try {
					Thread.sleep(500);
				} catch(Exception e) {System.err.println(e);}
			}
		}
	}
	
	/*
	 * image is responsible for graphically displaying the robot's position
	 */
	class Image extends JComponent {
		// eclipse complains at me if i don't have this
		private static final long serialVersionUID = 1L;

		public void paint(Graphics g) {
			double height = PlayingField.getBoundary().height;
			try{
				synchronized(penPoints){
					// draw the desired image
					g.setColor(Color.green);
					int dia =6;
					for(Point2D.Double p : dr.points) {
						g.fillOval((int)(p.x*scale)-dia/2, (int)(height - p.y*scale)-dia/2, dia, dia);
					}
					// draw the positions the robot has visited
					dia = 2;
					g.setColor(Color.red);
					for(Point2D.Double p : robotPoints) {
						g.fillOval((int)p.x-dia/2, (int)(height - p.y)-dia/2, dia, dia);
					}
					// draw the positions the pen has visited
					g.setColor(Color.blue);
					for(Point2D.Double p : penPoints) {
						g.fillOval((int)p.x-dia/2, (int)(height - p.y)-dia/2, dia, dia);
					}
				}
			} catch(Exception e) {System.err.println(e);}
		}
	}
}
