import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ErosionFilter extends EffectFilter
{	
	public ErosionFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Erosion " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		guiWindow.add(new JLabel("Erosion"));
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "Input";
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null)
			{
				if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
				{
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
				}
				
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				// we only use the red channel because the input is supposed 
				//  to be thresholded, so every pixel should be either pure 
				//  black (-1.0, -1.0, -1.0) or pure white (1.0, 1.0, 1.0): in
				//  either case, all the channels should have the same value.
				double[][] ired = temp.getChannelData(0);
				
				for(int i = 1; i < imgWidth-1; ++i)
				{
					for(int j = 1; j < imgHeight-1; ++j)
					{
						// Note that we use a cross-shaped erosion mask
						
						if(ired[i][j] > 0.9 && ired[i+1][j] > 0.9 && ired[i-1][j] > 0.9
							&& ired[i][j+1] > 0.9 && ired[i][j-1] > 0.9)
						{
							ored[i][j] = 1.0;
							ogreen[i][j] = 1.0;
							oblue[i][j] = 1.0;
						}
						else
						{
							ored[i][j] = -1.0;
							ogreen[i][j] = -1.0;
							oblue[i][j] = -1.0;
						}
					}
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}