import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;
import java.beans.*;

public class FilterDesktop implements Runnable, ActionListener, InternalFrameListener, PropertyChangeListener, ComponentListener
{
	private JFrame frame;
	private JDesktopPane desktop;
	private LinkDisplayDesktop linkDesktop;
	private ArrayList<EffectFilter> filters;
	private ArrayList<FilterLink> links;
	private static Thread theThread;
	private boolean isUpdating;
	
	private ArrayList<LinkArrow> arrows;
	
	private JFrame projectorOut;
	private FloatImageScreen projectorScreen;
	private DisplayFilter outputDisplay;
	
	public FilterDesktop()
	{
		filters = new ArrayList<EffectFilter>();
		links = new ArrayList<FilterLink>();
		isUpdating = true;
		
		arrows = null;
		
		frame = new JFrame("Filter System");
		//desktop = new JDesktopPane();
		linkDesktop = new LinkDisplayDesktop(this);
		desktop = linkDesktop;
		
		projectorOut = null;
		projectorScreen = null;
		
		setupMenus();
		
		frame.setSize(640, 480);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setContentPane(desktop);
		frame.setVisible(true);
		
		addFilter(new EffectFilter());
	}
	
	public void actionPerformed(ActionEvent e)
	{
		//System.out.println("Action!");
		
		String cmd = e.getActionCommand();
		
		if(cmd.equals("add_base"))
		{
			//Add another base filter
			addFilter(new EffectFilter());
		}
		if(cmd.equals("add_note"))
		{
			//Add a note filter
			addFilter(new NoteFilter());
		}
		if(cmd.equals("add_three_way"))
		{
			System.out.println("Adding three-way switch");
			addFilter(new ThreeWaySwitchFilter());
		}
		if(cmd.equals("add_ten_way"))
		{
			System.out.println("Adding ten-way switch");
			addFilter(new TenWaySwitchFilter());
		}
		else if(cmd.equals("add_display"))
		{
			System.out.println("Attempting to add display");
			addFilter(new DisplayFilter());
			// do something different
		}
		else if(cmd.equals("add_camera_input"))
		{
			System.out.println("Attempting to add camera input");
			addFilter(new CameraInputFilter());
		}
		else if(cmd.equals("add_threshold_a"))
		{
			System.out.println("Adding thresholdA");
			addFilter(new ThresholdAFilter());
		}
		else if(cmd.equals("add_distortion"))
		{
			System.out.println("Adding distortion");
			addFilter(new DistortionFilter());
		}
		else if(cmd.equals("add_scroll"))
		{
			System.out.println("Adding scroll filter");
			addFilter(new ScrollFilter());
		}
		else if(cmd.equals("add_random_color"))
		{
			System.out.println("Adding random color");
			addFilter(new RandomColorFilter());
		}
		else if(cmd.equals("add_particles"))
		{
			System.out.println("Adding particles");
			addFilter(new ParticleSpawnFilter());
		}
		else if(cmd.equals("add_skylight"))
		{
			System.out.println("Adding skylight");
			addFilter(new SkylightFilter());
		}
		else if(cmd.equals("add_chains"))
		{
			System.out.println("Adding chains");
			addFilter(new Chainer());
		}
		else if(cmd.equals("add_outline_grow"))
		{
			System.out.println("Adding outline grow");
			addFilter(new OutlineGrowFilter());
		}
		else if(cmd.equals("add_lens_correction"))
		{
			System.out.println("Adding lens correction");
			addFilter(new LensCorrectionFilter());
		}
		else if(cmd.equals("add_multiply"))
		{
			System.out.println("Adding multiply");
			addFilter(new MultiplyFilter());
		}
		else if(cmd.equals("add_zmultiply"))
		{
			System.out.println("Adding zmultiply");
			addFilter(new ZMultiplyFilter());
		}
		else if(cmd.equals("add_sine"))
		{
			System.out.println("Adding sine");
			addFilter(new SineFilter());
		}
		else if(cmd.equals("add_blur"))
		{
			System.out.println("Adding blur");
			addFilter(new BlurFilter());
		}
		else if(cmd.equals("add_bwmin"))
		{
			System.out.println("Adding bwmin");
			addFilter(new BWMinFilter());
		}
		else if(cmd.equals("add_simple_fire"))
		{
			System.out.println("Adding simpleFire");
			addFilter(new SimpleFireFilter());
		}
		else if(cmd.equals("add_cycler"))
		{
			System.out.println("Adding cycler");
			addFilter(new SimpleCyclerFilter());
		}
		else if(cmd.equals("add_delay"))
		{
			System.out.println("Adding delay");
			addFilter(new DelayFilter());
		}
		else if(cmd.equals("add_noise_reduction"))
		{
			System.out.println("Adding noise reduction");
			addFilter(new NoiseReductionFilter());
		}
		else if(cmd.equals("add_combined_calibration"))
		{
			System.out.println("Adding combined calibration");
			addFilter(new CombinedCalibrationFilter());
		}
		else if(cmd.equals("add_projector_transform"))
		{
			System.out.println("Adding projector transform");
			addFilter(new ProjectorTransformFilter());
		}
		else if(cmd.equals("add_manual_calibration"))
		{
			System.out.println("Adding manual calibration");
			addFilter(new ManualCalibrationFilter());
		}
		else if(cmd.equals("add_rotation"))
		{
			System.out.println("Adding rotation");
			addFilter(new ManualRotationFilter());
		}
		else if(cmd.equals("add_avg"))
		{
			System.out.println("Adding avg");
			addFilter(new AvgFilter());
		}
		else if(cmd.equals("add_composite"))
		{
			System.out.println("Adding composite");
			addFilter(new CompositeFilter());
		}
		else if(cmd.equals("add_cm"))
		{
			System.out.println("Adding center of mass");
			addFilter(new CenterMassFilter());
		}
		else if(cmd.equals("add_mover"))
		{
			System.out.println("Adding Image Mover");
			addFilter(new ImageMoveFilter());
		}
		else if(cmd.equals("add_shrink"))
		{
			System.out.println("Adding shrink");
			addFilter(new ShrinkFilter());
		}
		else if(cmd.equals("add_invert"))
		{
			System.out.println("Adding invert");
			addFilter(new InvertFilter());
		}
		else if(cmd.equals("add_upscale"))
		{
			System.out.println("Adding upscale");
			addFilter(new UpscaleFilter());
		}
		else if(cmd.equals("add_erosion"))
		{
			System.out.println("Adding erosion");
			addFilter(new ErosionFilter());
		}
		else if(cmd.equals("add_connected_components"))
		{
			System.out.println("Adding connected components");
			addFilter(new ConnectedComponentsFilter());
		}
		else if(cmd.equals("add_remove_small_components"))
		{
			System.out.println("Adding remove small components");
			addFilter(new RemoveSmallComponentsFilter());
		}
		else if(cmd.equals("add_body_segmentation"))
		{
			System.out.println("Adding body segmentation filter");
			addFilter(new BodySegmentationFilter());
		}
		else if(cmd.equals("add_image_loader"))
		{
			System.out.println("Adding image loader");
			addFilter(new ImageLoaderFilter());
		}
		else if(cmd.equals("add_movie_loader"))
		{
			System.out.println("Adding movie loader");
			addFilter(new MovieInputFilter());
		}
		else if(cmd.equals("add_multi_loader"))
		{
			System.out.println("Adding multi image loader");
			addFilter(new MultiImageLoaderFilter());
		}
		else if(cmd.equals("add_compute_average"))
		{
			System.out.println("Adding compute average");
			addFilter(new ComputeAverageFilter());
		}
		else if(cmd.equals("add_robot_power_light"))
		{
			System.out.println("Adding robot power light");
			addFilter(new RobotPowerLightFilter());
		}
		else if(cmd.equals("add_servo_control"))
		{
			System.out.println("Adding servo control");
			addFilter(new ServoControlFilter());
		}
		else if(cmd.equals("add_value_slider"))
		{
			System.out.println("Adding value slider");
			addFilter(new ValueSliderFilter());
		}
		else if(cmd.equals("add_robot_stub"))
		{
			System.out.println("Adding robot stub");
			addFilter(new RobotStubFilter());
		}
		else if(cmd.equals("add_red"))
		{
			System.out.println("Adding red filter");
			addFilter(new RedFilter());
		}
		else if(cmd.equals("add_hsv_conversion"))
		{
			System.out.println("Adding hsv conversion");
			addFilter(new HSVConversionFilter());
		}
		else if(cmd.equals("add_histogram_calculate"))
		{
			System.out.println("Adding histogram calculate");
			addFilter(new HistogramCalculatorFilter());
		}
		else if(cmd.equals("add_histogram_draw"))
		{
			System.out.println("Adding histogram draw");
			addFilter(new HistogramDrawFilter());
		}
		else if(cmd.equals("add_histogram_range"))
		{
			System.out.println("Adding histogram range");
			addFilter(new HistogramRangeFilter());
		}
		else if(cmd.equals("add_select_range"))
		{
			System.out.println("Adding select range");
			addFilter(new SelectRangeFilter());
		}
		else if(cmd.equals("add_data_capture"))
		{
			System.out.println("Adding data capture");
			addFilter(new DataCaptureFilter());
		}
		else if(cmd.equals("add_manual_range"))
		{
			System.out.println("Adding manual range");
			addFilter(new ManualRangeFilter());
		}
		else if(cmd.equals("add_post_identify_test"))
		{
			System.out.println("Adding post identify test");
			addFilter(new PostIdentifyTestFilter());
		}
		else if(cmd.equals("add_plane"))
		{
			System.out.println("Adding plane filter");
			addFilter(new PlaneFilter());
		}
		else if(cmd.equals("add_dot_locator"))
		{
			System.out.println("Adding dot locator");
			addFilter(new DotLocatorFilter());
		}
		else if(cmd.equals("add_optimized_select"))
		{
			System.out.println("Adding optimized select");
			addFilter(new OptimizedSelectFilter());
		}
	}
	
	public void setupMenus()
	{
		JMenuItem temp;
		
		JMenu mfile = new JMenu("File");
		temp = new JMenuItem("Save");
		temp.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				doSave();
			}
		});
		mfile.add(temp);
		
		temp = new JMenuItem("Load");
		temp.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				doLoad();
			}
		});
		mfile.add(temp);
		
		JMenu mfilters = new JMenu("Filters");
		
		JMenu inputFilters = new JMenu("Input");
		
		temp = new JMenuItem("CameraInput");
		temp.setActionCommand("add_camera_input");
		temp.addActionListener(this);
		inputFilters.add(temp);
		
		temp = new JMenuItem("ImageLoader");
		temp.setActionCommand("add_image_loader");
		temp.addActionListener(this);
		inputFilters.add(temp);
		
		temp = new JMenuItem("MovieLoader");
		temp.setActionCommand("add_movie_loader");
		temp.addActionListener(this);
		inputFilters.add(temp);
		
		temp = new JMenuItem("MultiImageLoader");
		temp.setActionCommand("add_multi_loader");
		temp.addActionListener(this);
		inputFilters.add(temp);
		
		JMenu robotFilters = new JMenu("Robot");
		
		temp = new JMenuItem("RobotPowerLight");
		temp.setActionCommand("add_robot_power_light");
		temp.addActionListener(this);
		robotFilters.add(temp);
		
		temp = new JMenuItem("ServoControl");
		temp.setActionCommand("add_servo_control");
		temp.addActionListener(this);
		robotFilters.add(temp);
		
		temp = new JMenuItem("PlaneTransform");
		temp.setActionCommand("add_plane");
		temp.addActionListener(this);
		robotFilters.add(temp);
		
		temp = new JMenuItem("OptimizedSelect");
		temp.setActionCommand("add_optimized_select");
		temp.addActionListener(this);
		robotFilters.add(temp);
		
		temp = new JMenuItem("DotLocator");
		temp.setActionCommand("add_dot_locator");
		temp.addActionListener(this);
		robotFilters.add(temp);
		
		temp = new JMenuItem("ValueSlider");
		temp.setActionCommand("add_value_slider");
		temp.addActionListener(this);
		robotFilters.add(temp);
		
		temp = new JMenuItem("RobotStub");
		temp.setActionCommand("add_robot_stub");
		temp.addActionListener(this);
		robotFilters.add(temp);
		
		JMenu computeFilters = new JMenu("Compute");
		
		temp = new JMenuItem("ComputeAverageFilter");
		temp.setActionCommand("add_compute_average");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		temp = new JMenuItem("RedFilter");
		temp.setActionCommand("add_red");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		temp = new JMenuItem("HSVConversion");
		temp.setActionCommand("add_hsv_conversion");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		temp = new JMenuItem("HistogramCalculate");
		temp.setActionCommand("add_histogram_calculate");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		temp = new JMenuItem("HistogramDraw");
		temp.setActionCommand("add_histogram_draw");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		temp = new JMenuItem("HistogramRange");
		temp.setActionCommand("add_histogram_range");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		temp = new JMenuItem("SelectRange");
		temp.setActionCommand("add_select_range");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		temp = new JMenuItem("DataCapture");
		temp.setActionCommand("add_data_capture");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		temp = new JMenuItem("ManualRange");
		temp.setActionCommand("add_manual_range");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		temp = new JMenuItem("PostIdentifyTest");
		temp.setActionCommand("add_post_identify_test");
		temp.addActionListener(this);
		computeFilters.add(temp);
		
		JMenu outputFilters = new JMenu("Output");
		
		temp = new JMenuItem("Display");
		temp.setActionCommand("add_display");
		temp.addActionListener(this);
		outputFilters.add(temp);
		
		outputFilters.add(new JMenuItem("ProjectorOut"));
		
		JMenu middleFilters = new JMenu("Effect");
		
		temp = new JMenuItem("BaseFilter");
		temp.setActionCommand("add_base");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("NoteFilter");
		temp.setActionCommand("add_note");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("ThreeWaySwitch");
		temp.setActionCommand("add_three_way");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("TenWaySwitch");
		temp.setActionCommand("add_ten_way");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("ThresholdA");
		temp.setActionCommand("add_threshold_a");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Distortion");
		temp.setActionCommand("add_distortion");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Scroll");
		temp.setActionCommand("add_scroll");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("RandomColor");
		temp.setActionCommand("add_random_color");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("ParticleSpawn");
		temp.setActionCommand("add_particles");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Skylight");
		temp.setActionCommand("add_skylight");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Chains");
		temp.setActionCommand("add_chains");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("OutlineGrow");
		temp.setActionCommand("add_outline_grow");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("LensCorrection");
		temp.setActionCommand("add_lens_correction");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Shrink");
		temp.setActionCommand("add_shrink");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Invert");
		temp.setActionCommand("add_invert");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Upscale");
		temp.setActionCommand("add_upscale");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Erosion");
		temp.setActionCommand("add_erosion");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("ConnectedComponents");
		temp.setActionCommand("add_connected_components");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("RemoveSmallComponents");
		temp.setActionCommand("add_remove_small_components");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("BodySegmentation");
		temp.setActionCommand("add_body_segmentation");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("WeightedAverage");
		temp.setActionCommand("add_avg");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Composite");
		temp.setActionCommand("add_composite");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Center of Mass");
		temp.setActionCommand("add_cm");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Image mover");
		temp.setActionCommand("add_mover");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("BWMin");
		temp.setActionCommand("add_bwmin");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("SimpleFire");
		temp.setActionCommand("add_simple_fire");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Cycler");
		temp.setActionCommand("add_cycler");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Delay");
		temp.setActionCommand("add_delay");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("NoiseReduction");
		temp.setActionCommand("add_noise_reduction");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("CombinedCalibration");
		temp.setActionCommand("add_combined_calibration");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("ProjectorTransform");
		temp.setActionCommand("add_projector_transform");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("ManualCalibration");
		temp.setActionCommand("add_manual_calibration");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("ManualRotation");
		temp.setActionCommand("add_rotation");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		temp = new JMenuItem("Sine");
		temp.setActionCommand("add_sine");
		temp.addActionListener(this);
		middleFilters.add(temp);
		
		JMenu middleFilters2 = new JMenu("Effect2");
		
		temp = new JMenuItem("Blur");
		temp.setActionCommand("add_blur");
		temp.addActionListener(this);
		middleFilters2.add(temp);
		
		temp = new JMenuItem("Multiply");
		temp.setActionCommand("add_multiply");
		temp.addActionListener(this);
		middleFilters2.add(temp);
		
		temp = new JMenuItem("ZMultiply");
		temp.setActionCommand("add_zmultiply");
		temp.addActionListener(this);
		middleFilters2.add(temp);
		
		mfilters.add(inputFilters);
		mfilters.add(outputFilters);
		mfilters.add(robotFilters);
		mfilters.add(computeFilters);
		mfilters.add(middleFilters);
		mfilters.add(middleFilters2);
		
		temp = new JMenuItem("Make Connection");
		temp.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				makeConnection();
			}
		});
		
		JMenu linkages = new JMenu("Links");
		linkages.add(temp);
		
		temp = new JCheckBoxMenuItem("Draw Links", true);
		temp.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				if(e != null)
				{
					linkDesktop.setLinkDrawing(((JCheckBoxMenuItem)e.getSource()).isSelected());
				}
			}
		});
		
		linkages.add(temp);
		
		temp = new JMenuItem("Projector Output");
		temp.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				selectProjectorOutput();
			}
		});
		
		linkages.add(temp);
		
		temp = new JMenuItem("Rename Filter");
		temp.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				renameFilter();
			}
		});
		
		linkages.add(temp);
		
		JMenuBar menus = new JMenuBar();
		menus.add(mfile);
		menus.add(mfilters);
		menus.add(linkages);
		frame.setJMenuBar(menus);
	}
	
	public void doSave()
	{
		JFileChooser chooser = new JFileChooser();
		int result = chooser.showSaveDialog(desktop);
		if(result == JFileChooser.APPROVE_OPTION)
		{
			File selected = chooser.getSelectedFile();
			if(selected != null)
				saveDesktop(selected);
		}
	}
	
	public void doLoad()
	{
		JFileChooser chooser = new JFileChooser();
		int result = chooser.showOpenDialog(desktop);
		if(result == JFileChooser.APPROVE_OPTION)
		{
			File selected = chooser.getSelectedFile();
			if(selected != null)
				loadDesktop(selected);
		}
	}
	
	public void saveDesktop(File dest)
	{
		for(int i = 0; i < filters.size(); ++i)
		{
			filters.get(i).prepareForSave();
		}
		
		try
		{
			FileOutputStream fOut = new FileOutputStream(dest);
			ObjectOutputStream out = new ObjectOutputStream(fOut);
			out.writeObject(filters); 
		}
		catch(Exception e)
		{
			System.out.println(e);
		}
	}
	
	public void loadDesktop(File src)
	{	
		//System.out.println("Interrupting...");
	
		//theThread.interrupt();
		isUpdating = false;
		
		// I don't think this is necessary, but it works, and I don't want
		//   to mess with it.
		try
		{
			Thread.sleep(500);
		}
		catch(InterruptedException e)
		{
			// do nothing
		}
		
		ArrayList<EffectFilter> tempFilters;
	
		try
		{
			FileInputStream fIn = new FileInputStream(src);
			ObjectInputStream in = new ObjectInputStream(fIn);
			tempFilters = (ArrayList<EffectFilter>)in.readObject();
			desktop.removeAll();
			
			for(int i = 0; i < tempFilters.size(); ++i)
			{
				tempFilters.get(i).rebuildFromSave();
				
				// next we add the filter's window to the gui desktop
				desktop.add(tempFilters.get(i).getWindow());
			
				// finally, we make the desktop listen to the window events
				tempFilters.get(i).getWindow().addInternalFrameListener(this);
				
				// additionally, we want it to recieve move and resize events
				tempFilters.get(i).getWindow().addComponentListener(this);
			}
			
			filters = tempFilters;
		}
		catch(Exception e)
		{
			System.out.println(e);
		}
		
		rebuildLinks();
		
		desktop.revalidate();
		desktop.repaint();
		
		isUpdating = true;
		
		//theThread.run();
	}
	
	public synchronized void rebuildLinks()
	{
		links = new ArrayList<FilterLink>();
		FilterLink[] inputs;
		
		for(int i = 0; i < filters.size(); ++i)
		{
			inputs = filters.get(i).getInputs();
			if(inputs != null)
			{
				for(int j = 0; j < inputs.length; ++j)
				{
					links.add(inputs[j]);
				}
			}
		}
		
		arrows = null;
	}
	
	public synchronized ArrayList<LinkArrow> getArrows()
	{
		if(arrows != null)
			return arrows;
		else
		{
			// build arrows
			FilterLink temp;
			
			arrows = new ArrayList<LinkArrow>();
			
			for(int i = 0; i < links.size(); ++i)
			{
				temp = links.get(i);
				
				if(temp != null && temp.getSource() != null && temp.getDest() != null)
				{
					arrows.add(new LinkArrow(temp.getSource().getWindow(), temp.getDest().getWindow()));
				}
			}
			
			return null;
		}
	}
	
	public synchronized void changeLink(FilterLink newLink)
	{
		// there should be at most one link that already
		// connects to the newLink's input: we must delete that one
		
		FilterLink temp;
		
		for(int i = 0; i < links.size(); ++i)
		{
			temp = links.get(i);
			
			if(temp != null && temp.getDest() == newLink.getDest() && temp.getDestChannel() == newLink.getDestChannel())
			{
				links.remove(i);
				break;
			}
		}
		links.add(newLink);
		
		arrows = null;
	}
	
	public void makeConnection()
	{
		ArrayList<IOIdentifier> outputs = new ArrayList<IOIdentifier>();
		ArrayList<IOIdentifier> inputs = new ArrayList<IOIdentifier>();
		ArrayList<String> outputNames = new ArrayList<String>();
		ArrayList<String> inputNames = new ArrayList<String>();
		
		int numFilters = filters.size();
		
		EffectFilter tempFilter;
		
		// First, we create lists of links to input and output channels, along
		//   with arrays of Strings representing their names
		for(int i = 0; i < numFilters; ++i)
		{
			tempFilter = filters.get(i);
			String temp;
			
			// Now we enumerate inputs
			for(int j = 0; j < tempFilter.numInputs(); ++j)
			{
				inputs.add(new IOIdentifier(tempFilter, j));
				temp = tempFilter.getName() + ":" + tempFilter.getInputName(j);
				inputNames.add(temp);
			}
			
			// Now we enumerate outputs
			for(int k = 0; k < tempFilter.numOutputs(); ++k)
			{
				outputs.add(new IOIdentifier(tempFilter, k));
				temp = tempFilter.getName() + ":" + tempFilter.getOutputName(k);
				outputNames.add(temp);
			}
		}
		
		// Now, we create combo boxes to be able to choose from them
		JComboBox inputSelection = new JComboBox(inputNames.toArray(new String[0]));
		JComboBox outputSelection = new JComboBox(outputNames.toArray(new String[0]));
		inputSelection.setEditable(false);
		outputSelection.setEditable(false);
		
		// Now, we show a dialog box to let the user select from these
		int result = JOptionPane.showOptionDialog(frame, 
					new Object[]{inputSelection, outputSelection},
					"Make a connection", JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE, null, null, null);
					
		// If the user clicked 'OK', then we attempt to make the connection
		int selectedOutput = outputSelection.getSelectedIndex();
		int selectedInput = inputSelection.getSelectedIndex();
		
		if(result == JOptionPane.OK_OPTION && selectedOutput != -1 && selectedInput != -1)
		{
			System.out.println("Attempting to make connection");
			
			FilterLink tempLink = new FilterLink(outputs.get(selectedOutput).filt, 
												inputs.get(selectedInput).filt,
												outputs.get(selectedOutput).chan,
												inputs.get(selectedInput).chan);
												
			inputs.get(selectedInput).filt.setInput(inputs.get(selectedInput).chan, tempLink);
			
			changeLink(tempLink);
		}
	}
	
	public void selectProjectorOutput()
	{
		ArrayList<DisplayFilter> displays = new ArrayList<DisplayFilter>();
		ArrayList<String> displayNames = new ArrayList<String>();
		
		EffectFilter tempFilter;
		
		for(int i = 0; i < filters.size(); ++i)
		{
			tempFilter = filters.get(i);
			if(tempFilter instanceof DisplayFilter)
			{
				displays.add((DisplayFilter)tempFilter);
				displayNames.add(tempFilter.getName());
			}	
		}
		
		JComboBox displaySelection = new JComboBox(displayNames.toArray(new String[0]));
		
		int result = JOptionPane.showOptionDialog(frame, new Object[]{displaySelection},
						"Select a display to project", JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE, null, null, null);
						
		int selectedDisplayIndex = displaySelection.getSelectedIndex();
		
		if(result == JOptionPane.OK_OPTION && selectedDisplayIndex != -1)
		{
			JFrame tempFrame = new JFrame("Projector Out");
			projectorScreen = new FloatImageScreen();
			
			outputDisplay = displays.get(selectedDisplayIndex);
			
			tempFrame.getContentPane().add(projectorScreen);
			projectorOut = tempFrame;
			projectorOut.setSize(640,480);
			projectorOut.setVisible(true);
			
		}
		
	}
	
	public void renameFilter()
	{
		ArrayList<String> filterNames = new ArrayList<String>();
		
		EffectFilter tempFilter;
		
		for(int i = 0; i < filters.size(); ++i)
		{
			tempFilter = filters.get(i);
			filterNames.add(tempFilter.getName());	
		}
		
		JComboBox filterSelection = new JComboBox(filterNames.toArray(new String[0]));
		JTextArea filterName = new JTextArea();
		filterName.setEditable(true);
		
		int result = JOptionPane.showOptionDialog(frame, new Object[]{filterSelection, filterName},
						"Rename a filter", JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE, null, null, null);
						
		int selectedFilterIndex = filterSelection.getSelectedIndex();
		
		if(result == JOptionPane.OK_OPTION && selectedFilterIndex != -1)
		{
			filters.get(selectedFilterIndex).setName(filterName.getText());	
		}
		
	}
	
	public void addFilter(EffectFilter addee)
	{
		// first we add the filter to the filter array
		filters.add(addee);
		
		// next we add the filter's window to the gui desktop
		desktop.add(addee.getWindow());
		
		// finally, we make the desktop listen to the window events
		addee.getWindow().addInternalFrameListener(this);
		
		// make sure we get window property change events as well
		//addee.getWindow().addPropertyChangeListener(this);
		
		// we want to receive window resize and move events
		addee.getWindow().addComponentListener(this);
	}
	
	public void removeFilter(EffectFilter removee)
	{
		// First, we find the actual filter
		int numThings = filters.size();
		for(int i = 0; i < numThings; ++i)
		{
			if(filters.get(i) == removee)
			{
				removeFilter(i);
				break;
			}
		}
	}
	
	public void removeFilter(int n)
	{
		EffectFilter removee = filters.get(n);
		
		filters.remove(n);
		
		// Next, we remove its window, if it exists
		Component[] iWindows = desktop.getComponents();
		for(int i = 0; i < iWindows.length; ++i)
		{
			if(iWindows[i] == removee.getWindow())
			{
				desktop.remove(removee.getWindow());
				break;
			}
		}
	}
	
	// InternalFrameListener stuff
	public void internalFrameActivated(InternalFrameEvent e)
	{
		desktop.repaint();
	}
	
	public void internalFrameClosed(InternalFrameEvent e)
	{
		System.out.println("Window closed");
		for(int i = 0; i < filters.size(); ++i)
		{
			if(filters.get(i).getWindow() == e.getInternalFrame())
			{
				removeFilter(i);
				break;
			}
		}
		rebuildLinks();
	}
	
	public void internalFrameClosing(InternalFrameEvent e)
	{
	}
	
	public void internalFrameDeactivated(InternalFrameEvent e)
	{
	}
	
	public void internalFrameDeiconified(InternalFrameEvent e)
	{
	}
	
	public void internalFrameIconified(InternalFrameEvent e)
	{
	}
	
	public void internalFrameOpened(InternalFrameEvent e)
	{
	}
	
	public void propertyChange(PropertyChangeEvent evt)
	{
		//RepaintManager.currentManager(null).markCompletelyDirty(desktop);
		System.out.println("Property changed");
		desktop.repaint();
	}
	
	public void componentHidden(ComponentEvent e)
	{
	}
	
	public void componentMoved(ComponentEvent e)
	{
		desktop.repaint();
	}
	
	public void componentResized(ComponentEvent e)
	{
		desktop.repaint();
	}
	
	public void componentShown(ComponentEvent e)
	{
	}
	
	
	public void run()
	{
		// Run forever, trying to update all the things...
		while(true)
		{
			try{
				//System.out.println("Running");
				if(isUpdating)
				{
					for(int i = 0; i < filters.size(); ++i)
					{
						filters.get(i).update(0);
					}
					if(projectorOut != null && projectorScreen != null && outputDisplay != null) 
					{
						projectorScreen.imgUpdate(outputDisplay.getImage());
					}
				}
				Thread.sleep(1);
			}
			catch(InterruptedException e)
			{
				// do nothing...
			}
		}
	}
	
	public static void main(String[] args)
	{
		// All main does is instantiate the desktop
		FilterDesktop theDesktop = new FilterDesktop();
		theThread = new Thread(theDesktop);
		theThread.start();
	}
}