import java.io.*;

public class FloatImage implements Serializable
{
	private FloatChannel[] channels;
	private int imageWidth;
	private int imageHeight;
	
	private static final int DEFAULT_CHANNELS = 3;
	private static final int DEFAULT_WIDTH = 160;
	private static final int DEFAULT_HEIGHT = 120;
	
	public FloatImage()
	{
		channels = new FloatChannel[DEFAULT_CHANNELS];
		imageWidth = DEFAULT_WIDTH;
		imageHeight = DEFAULT_HEIGHT;
		
		for(int i = 0; i < DEFAULT_CHANNELS; ++i)
		{
			channels[i] = new FloatChannel(imageWidth, imageHeight);
		}
	}
	
	public FloatImage(int numChannels, int width, int height)
	{
		channels = new FloatChannel[numChannels];
		imageWidth = width;
		imageHeight = height;
		
		for(int i = 0; i < numChannels; ++i)
		{
			channels[i] = new FloatChannel(imageWidth, imageHeight);
		}
	}
	
	public double[][] getChannelData(int channel)
	{
		if(channels != null && channels[channel] != null)
		{
			return channels[channel].getData();
		}
		return null;
	}
	
	public void remake(int newChannels, int newWidth, int newHeight)
	{
		channels = new FloatChannel[newChannels];
		imageWidth = newWidth;
		imageHeight = newHeight;
		
		for(int i = 0; i < newChannels; ++i)
		{
			channels[i] = new FloatChannel(imageWidth, imageHeight);
		}	
	}
	
	public boolean copyFrom(FloatImage src)
	{
		if(src.getWidth() != getWidth() || src.getHeight() != getHeight()
			|| src.numChannels() != numChannels())
			return false;
			
		double[][] ichan, ochan;
			
		for(int k = 0; k < channels.length; ++k)
		{
			ichan = src.getChannelData(k);
			ochan = getChannelData(k);
			
			for(int i = 0; i < imageWidth; ++i)
			{
				for(int j = 0; j < imageHeight; ++j)
				{
					ochan[i][j] = ichan[i][j];
				}
			}	
		}
			
		return true;
	}
	
	public FloatImage duplicate()
	{
		FloatImage temp = new FloatImage(channels.length, imageWidth, imageHeight);
		
		double[][] ichan, ochan;
		
		for(int k = 0; k < channels.length; ++k)
		{
			ichan = getChannelData(k);
			ochan = temp.getChannelData(k);
			
			for(int i = 0; i < imageWidth; ++i)
			{
				for(int j = 0; j < imageHeight; ++j)
				{
					ochan[i][j] = ichan[i][j];
				}
			}
		}
		
		return temp;
	}
	
	public void zeroOut()
	{
		double[][] ochan;
		
		for(int k = 0; k < channels.length; ++k)
		{
			ochan = getChannelData(k);
			
			for(int i = 0; i < imageWidth; ++i)
			{
				for(int j = 0; j < imageHeight; ++j)
				{
					ochan[i][j] = -1.0;
				}
			}
		}
	}
	
	public int numChannels()
	{
		return channels.length;
	}
	
	public int getWidth()
	{
		return imageWidth;
	}
	
	public int getHeight()
	{
		return imageHeight;
	}
}