import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class HistogramRangeFilter extends EffectFilter
{	
	public static final double MIN_FRAC = 0.1;
	public static final double MAX_FRAC = 0.9;

	public HistogramRangeFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Histogram Range " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage(3, 2, 1);
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		guiWindow.add(new JLabel("Histogram Range"));
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "DATA: Histogram In";
	}
	
	public String getOutputName(int outputNum)
	{
		return "DATA: Range Out";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		if(internalBuffer == null || internalBuffer.getWidth() != 2 ||
			internalBuffer.getHeight() != 1)
			internalBuffer = new FloatImage(3, 2, 1);
			
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			// First, calculate the total number of points
			double totalFreq = 0.0;
			double[][] rdata = temp.getChannelData(0);
			for(int i = 0; i < temp.getWidth(); ++i)
				totalFreq += rdata[i][0];
				
			double minAmount = totalFreq * MIN_FRAC;
			double maxAmount = totalFreq * MAX_FRAC;
			
			double[][] rangeOut;
			double[][] histIn;
			
			double binMult = 2.0 / (double)temp.getWidth();
			
			for(int k = 0; k < 3; ++k)
			{
				rangeOut = internalBuffer.getChannelData(k);
				histIn = temp.getChannelData(k);
				
				if(k >= 1)
				{
					rangeOut[0][0] = -1.0;
					rangeOut[1][0] = 1.0;
					continue;
				}
				
				int minStart = 0;
				int maxEnd = 0;
				double sumSoFar = 0.0;
				
				int i = 0;
				while(i < temp.getWidth() && sumSoFar < minAmount)
				{
					sumSoFar += histIn[i][0];
					++i;
				}
				minStart = i;
				while(i < temp.getWidth() && sumSoFar < maxAmount)
				{
					sumSoFar += histIn[i][0];
					++i;
				}
				maxEnd = i;
				
				rangeOut[0][0] = (((double)minStart) * binMult) - 1.0 - 0.2;
				rangeOut[1][0] = (((double)maxEnd) * binMult) - 1.0 + 0.2;
			}
			
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}