import java.util.ArrayList;

class PPoint
{
	public double x;
	public double y;
	
	public PPoint(double x, double y)
	{
		this.x = x;
		this.y = y;
	}
	
	public double distanceTo(PPoint rhs)
	{
		double dx = x - rhs.x;
		double dy = y - rhs.y;
		return Math.sqrt(dx*dx + dy*dy);
	}
}

class DistanceObj implements ObjectiveFunction
{
	protected ArrayList<TrackingPost> observedDistances;
	protected PPoint[] knownPosts;
	
	public DistanceObj(ArrayList<TrackingPost> dists, PPoint[] knows)
	{
		observedDistances = dists;
		knownPosts = knows;
	}
	
	public double evaluate(double x, double y)
	{
		TrackingPost curPost;
		PPoint refPoint;
		
		double error = 0.0;
		double dx, dy, distance, ddist;
		
		for(int i = 0; i < observedDistances.size(); ++i)
		{
			curPost = observedDistances.get(i);
			
			if(curPost.postNum < 0 || curPost.postNum >= 4)
				continue;
			
			refPoint = knownPosts[curPost.postNum];
			
			dx = x - refPoint.x;
			dy = y - refPoint.y;
			distance = Math.sqrt(dx*dx + dy*dy);
			ddist = distance - curPost.distance;
			
			error += (ddist * ddist);
		}
		
		return error;
	}
}

public class LocationUtils
{
	protected static PPoint[] _posts;
	protected static boolean _postsSetup = false;
	
	protected final static double 	SEARCH_SX = 91.0;
	protected final static double 	SEARCH_SY = 91.0;
	protected final static double 	SEARCH_SR = 100.0;
	protected final static int		SEARCH_BRANCH = 40;
	protected final static int		SEARCH_LEVELS = 3;
	
	public static void setupPosts()
	{
		_posts = new PPoint[4];
		_posts[0] = new PPoint(0.0, 0.0);
		_posts[1] = new PPoint(182.88, 0.0);
		_posts[2] = new PPoint(182.88, 182.88);
		_posts[3] = new PPoint(0.0, 182.88);
		
		_postsSetup = true;
	}
	
	public static PPoint angleLocalize(ArrayList<TrackingPost> posts)
	{
		return null;
	}
	
	public static PPoint distanceLocalize(ArrayList<TrackingPost> posts)
	{
		if(!_postsSetup)
			setupPosts();
		
		DistanceObj f = new DistanceObj(posts, _posts);
		return objhelper(new PPoint(SEARCH_SX, SEARCH_SY), SEARCH_SR, SEARCH_LEVELS, f);
	}
	
	public static double calculateTheta(TrackingPost srcPost, PPoint location)
	{
		return angleBetween(location, _posts[srcPost.postNum]) + srcPost.cx;
	}
	
	public static double angleBetween(PPoint p1, PPoint p2)
	{
		double dx = p2.x - p1.x;
		// we have to flip this due to the strange coordinate system...
		double dy = p1.y -  p2.y;
		
		double adx = Math.abs(dx);
		double ady = Math.abs(dy);
		
		double theta;
		
		if(ady < adx)
			theta = Math.atan(ady / adx);
		else
			theta = (Math.PI / 2.0) - Math.atan(adx / ady);
			
		if(dx >= 0.0 && dy >= 0.0)
			theta = theta;
		else if(dx < 0.0 && dy >= 0.0)
			theta = Math.PI - theta;
		else if(dx < 0.0 && dy < 0.0)
			theta = Math.PI + theta;
		else if(dx >= 0.0 && dy < 0.0)
			theta = (2.0 * Math.PI) - theta;
			
		System.out.println("Calculated roomba-to-post theta: " + ((theta * 180.0) / Math.PI));
			
		return theta;
	}
	
	public static PPoint objhelper(PPoint searchCenter, double searchRad, int depth, ObjectiveFunction f)
	{
		double sx = searchCenter.x - searchRad;
		double sy = searchCenter.y - searchRad;
		
		double si = (searchRad * 2.0) / (double)SEARCH_BRANCH;
		
		double x = sx;
		double y = sy;
		
		double val;
		
		PPoint bestPoint = new PPoint(0.0, 0.0);
		double bestVal = 1000000.0;
		
		for(int i = 0; i < SEARCH_BRANCH; ++i)
		{
			y = sy;
			for(int j = 0; j < SEARCH_BRANCH; ++j)
			{
				val = f.evaluate(x, y);
				if(val < bestVal)
				{
					bestVal = val;
					bestPoint.x = x;
					bestPoint.y = y;
				}
				y += si;
			}
			x += si;
		}
		
		if(depth <= 0)
			return bestPoint;
		else
			return objhelper(bestPoint, searchRad / (double)SEARCH_BRANCH, depth-1, f);
	}
}