import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ManualRangeFilter extends EffectFilter
{	
	private transient JSlider[][] sliders;
	private double[][] sliderVals;

	public ManualRangeFilter()
	{
		filterInputs = new FilterLink[0];
		dead = false;
		filterName = "Manual Range Filter " + (int)(Math.random() * 1000);
		
		sliderVals = new double[3][2];
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		getSliderVals();
	}
	
	private void getSliderVals()
	{
		for(int i = 0; i < 3; ++i)
		{
			for(int j = 0; j < 2; ++j)
			{
				sliderVals[i][j] = ((double)sliders[i][j].getValue()) / 100.0;
			}
		}
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		JPanel whatever = new JPanel();
		
		sliders = new JSlider[3][2];
		
		JSlider curSlider;
		
		curSlider = new JSlider(JSlider.HORIZONTAL);
		curSlider.setMajorTickSpacing(10);
		curSlider.setMinorTickSpacing(5);
		curSlider.setPaintTicks(true);
		curSlider.setBorder(BorderFactory.createTitledBorder("Ch.1 Min"));
		curSlider.setMinimum(-100);
		curSlider.setMaximum(100);
		curSlider.setValue((int)(sliderVals[0][0] * 100.0));
		sliders[0][0] = curSlider;
		whatever.add(curSlider);
		
		curSlider = new JSlider(JSlider.HORIZONTAL);
		curSlider.setMajorTickSpacing(10);
		curSlider.setMinorTickSpacing(5);
		curSlider.setPaintTicks(true);
		curSlider.setBorder(BorderFactory.createTitledBorder("Ch.1 Max"));
		curSlider.setMinimum(-100);
		curSlider.setMaximum(100);
		curSlider.setValue((int)(sliderVals[0][1] * 100.0));
		sliders[0][1] = curSlider;
		whatever.add(curSlider);
		
		curSlider = new JSlider(JSlider.HORIZONTAL);
		curSlider.setMajorTickSpacing(10);
		curSlider.setMinorTickSpacing(5);
		curSlider.setPaintTicks(true);
		curSlider.setBorder(BorderFactory.createTitledBorder("Ch.2 Min"));
		curSlider.setMinimum(-100);
		curSlider.setMaximum(100);
		curSlider.setValue((int)(sliderVals[1][0] * 100.0));
		sliders[1][0] = curSlider;
		whatever.add(curSlider);
		
		curSlider = new JSlider(JSlider.HORIZONTAL);
		curSlider.setMajorTickSpacing(10);
		curSlider.setMinorTickSpacing(5);
		curSlider.setPaintTicks(true);
		curSlider.setBorder(BorderFactory.createTitledBorder("Ch.2 Max"));
		curSlider.setMinimum(-100);
		curSlider.setMaximum(100);
		curSlider.setValue((int)(sliderVals[1][1] * 100.0));
		sliders[1][1] = curSlider;
		whatever.add(curSlider);
		
		curSlider = new JSlider(JSlider.HORIZONTAL);
		curSlider.setMajorTickSpacing(10);
		curSlider.setMinorTickSpacing(5);
		curSlider.setPaintTicks(true);
		curSlider.setBorder(BorderFactory.createTitledBorder("Ch.3 Min"));
		curSlider.setMinimum(-100);
		curSlider.setMaximum(100);
		curSlider.setValue((int)(sliderVals[2][0] * 100.0));
		sliders[2][0] = curSlider;
		whatever.add(curSlider);
		
		curSlider = new JSlider(JSlider.HORIZONTAL);
		curSlider.setMajorTickSpacing(10);
		curSlider.setMinorTickSpacing(5);
		curSlider.setPaintTicks(true);
		curSlider.setBorder(BorderFactory.createTitledBorder("Ch.3 Max"));
		curSlider.setMinimum(-100);
		curSlider.setMaximum(100);
		curSlider.setValue((int)(sliderVals[2][1] * 100.0));
		sliders[2][1] = curSlider;
		whatever.add(curSlider);
		
		guiWindow.add(whatever);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "Input";
	}
	
	public String getOutputName(int outputNum)
	{
		return "DATA: Range out";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		
		if(needsProcessing)
		{
			getSliderVals();
			
			if(internalBuffer == null || internalBuffer.getWidth() != 2 ||
				internalBuffer.getHeight() != 1)
				internalBuffer = new FloatImage(3, 2, 1);
				
			double[][] outData;	
			for(int k = 0; k < 3; ++k)
			{
				outData = internalBuffer.getChannelData(k);
				outData[0][0] = sliderVals[k][0];
				outData[1][0] = sliderVals[k][1];
			}
			
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}