public class MotionModel {
	private static final double dPosition = .1;
	private static final double dTheta = Math.PI/128;
	
	private double deltaR, turn1, turn2;
	
	public MotionModel(double deltaX, double deltaY, double deltaTheta) {
	
		deltaR = Math.sqrt(deltaX*deltaX + deltaY*deltaY);
		turn1 = Math.atan2(deltaY, deltaX);
		turn2 = -turn1 + deltaTheta;
	}
	
	public MotionModel(double deltaD, double deltaTheta) { 
		if (deltaTheta == 0.0) {
			turn1 = 0.0;
			turn2 = 0.0;
			deltaR = deltaD;
		} else {
			turn1 = deltaTheta / 2.0;
			turn2 = deltaTheta / 2.0;
			deltaR = (deltaD * Math.sin(deltaTheta))/(deltaTheta * Math.sin((Math.PI - deltaTheta)/2.0));
		}
	} 
	
	/**
	 * apply the specified motion to a particle p
	 */
	public void applyMotion(Particle p) {
		//System.out.println("moving..");
        double randTurn1 = PlayingField.randGaussian()*dTheta + turn1;
        double randTurn2 = PlayingField.randGaussian()*dTheta + turn2;
        double randDeltaR = PlayingField.randGaussian()*dPosition*deltaR + deltaR;
        p.x += randDeltaR*Math.cos(p.theta + randTurn1);
        p.y += randDeltaR*Math.sin(p.theta + randTurn1);          
        p.theta = PlayingField.normalizeAngle(p.theta + randTurn1 + randTurn2);
	}
	
	/**
	 * apply motion without any randomness
	 */
	public void exactMotion(Particle p) {
		p.x += deltaR*Math.cos(p.theta + turn1);
		p.y += deltaR*Math.sin(p.theta + turn1);
		p.theta = PlayingField.normalizeAngle(p.theta + turn1 + turn2);
	}
}





