import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.io.*;
import javax.media.*;
import javax.media.datasink.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.media.util.*;
import javax.media.control.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.net.URL;
import com.sun.media.protocol.vfw.VFWCapture;

public class MovieInputFilter extends EffectFilter implements ComponentListener, InternalFrameListener
{
    protected final static int MIN_WIDTH  = 100;  // 320;
    protected final static int MIN_HEIGHT = 100;  // 240;
 
    protected static int shotCounter = 1;
 
    protected transient JLabel statusBar = null;
    protected transient JPanel visualContainer = null;
    protected transient Component visualComponent = null;
    protected transient Component videoControls = null;
    protected transient JToolBar toolbar = null;
    protected transient MyToolBarAction loadButton    = null;
    protected transient MyToolBarAction captureButton   = null;
 
    protected transient Player player = null;
    protected transient CaptureDeviceInfo webCamDeviceInfo = null;
    protected transient MediaLocator ml = null;
    protected transient Dimension imageSize = null;
    protected transient FormatControl formatControl = null;
 
    protected transient VideoFormat currentFormat = null;
    protected transient Format[] videoFormats = null;
 
    protected boolean initialised = false;
		
	public MovieInputFilter()
	{
		filterInputs = null;
		dead = false;
		filterName = "Movie Input";
		
		guiWidth = 320;
		guiHeight = 260;
		guiX = 50;
		guiY = 50;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		//super.rebuildFromSave();
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
        guiWindow.addInternalFrameListener ( this );
        guiWindow.addComponentListener ( this );
 
        guiWindow.setLayout ( new BorderLayout() );
 
        visualContainer = new JPanel();
        visualContainer.setLayout ( new BorderLayout() );
 
        guiWindow.add ( visualContainer, BorderLayout.CENTER );
 
 		/*
        statusBar = new JLabel ("")
        {
            // Nasty bug workaround
            // The minimum JLabel size was determined by the text in the status bar
            // So the layoutmanager wouldn't shrink the window for the video image
            public Dimension getPreferredSize (  )
            {
                // get the JLabel to "allow" a minimum of 10 pixels width
                // not to work out the minimum size from the text length
                return ( new Dimension ( 10, super.getPreferredSize().height ) );
            }
        };
 
        statusBar.setBorder ( new EtchedBorder() );
        guiWindow.add ( statusBar, BorderLayout.SOUTH );
        */
		
		try
		{
			initialise();
		}
		catch(Exception e)
		{
			System.out.println("Super bad error");
		}
		
		guiWindow.setVisible(true);
		
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "NONE";
	}
	
	public String getOutputName(int outputNum)
	{
		return "MovieOut";
	}
	
	public int numInputs()
	{
		return 0;
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		final int MASK = 0xFF;
		
		if(needsProcessing && player != null)
		{
			Image photo = grabFrameImage ( );
	        if ( photo != null )
	        {
	                //MySnapshot snapshot = new MySnapshot ( photo, new Dimension ( imageSize ) );
	            BufferedImage cap = createBufferedImage(photo);
	            
	            if(cap.getWidth() != internalBuffer.getWidth() || cap.getHeight() != internalBuffer.getHeight())
	            {
	            	internalBuffer = new FloatImage(3, cap.getWidth(), cap.getHeight());
	            }
	                
				double[][] red = internalBuffer.getChannelData(0);
				double[][] green = internalBuffer.getChannelData(1);
				double[][] blue = internalBuffer.getChannelData(2);
				
				int bred, bgreen, bblue, brgb;
			
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
			
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						brgb = cap.getRGB(i, j);
						bblue = brgb & MASK;
						bgreen = (brgb >> 8) & MASK;
						bred = (brgb >> 16) & MASK;
						
						red[i][j] = (((double)bred) / 128.0) - 1.0;
						green[i][j] = (((double)bgreen) / 128.0) - 1.0;
						blue[i][j] = (((double)bblue) / 128.0) - 1.0;
					}
				}
	        }
	        needsProcessing = false;
        }

		return internalBuffer;
	}
	
    /* --------------------------------------------------------------
     * Initialise
     *
     * @initializes
     * -------------------------------------------------------------- */
 
    public boolean initialise ( )
        throws Exception
    {
         setUpToolBar();
         guiWindow.add ( toolbar, BorderLayout.NORTH );
         return true;
    }
 
    protected void setUpToolBar ( )
    {
        toolbar = new JToolBar();
 
 
        // Note : due to cosmetic glitches when undocking and docking the toolbar,
        //        I've set this to false.
        toolbar.setFloatable(false);
 
        // Note : If you supply the 16 x 16 bitmaps then you can replace
        // the commented line in the MyToolBarAction constructor
 
        loadButton    = new MyToolBarAction ( "Load", "BtnFormat.jpg" );
        captureButton   = new MyToolBarAction ( "Capture",    "BtnCapture.jpg" );
 
        toolbar.add ( loadButton );
        toolbar.add ( captureButton );
 
        guiWindow.add ( toolbar, BorderLayout.NORTH );
    }
 
 
    protected void toolbarHandler ( MyToolBarAction actionBtn )
    {
        if ( actionBtn == loadButton )
        {
        	doMovieLoadDialog();
        }
        else if ( actionBtn == captureButton )
        {
            Image photo = grabFrameImage ( );
            if ( photo != null )
            {
                //MySnapshot snapshot = new MySnapshot ( photo, new Dimension ( imageSize ) );
            }
            else
            {
                System.err.println ("Error : Could not grab frame");
            }
        }
    }
    
	protected void doMovieLoadDialog()
	{
		JFileChooser chooser = new JFileChooser();
		int result = chooser.showOpenDialog(guiWindow);
		if(result == JFileChooser.APPROVE_OPTION)
		{
			File selected = chooser.getSelectedFile();
			try
			{
				if(selected != null)
				{
					URL url = new URL("file:" + selected.getAbsolutePath());
					
					// remove the old  player stuff if necessary
					//  (if it exists)
					if(player != null)
					{
						player.stop();
						visualContainer.remove(visualComponent);
						visualContainer.remove(videoControls);
					}
					
					player = Manager.createRealizedPlayer(url);
					
					visualComponent = player.getVisualComponent();
					videoControls = player.getControlPanelComponent();
					
					visualContainer.add(videoControls, BorderLayout.SOUTH);
					
					player.start();
					
	                if ( visualComponent != null )
	                {
	                    visualContainer.add ( visualComponent, BorderLayout.CENTER );
	
	                    guiWindow.invalidate();
	                    guiWindow.pack();
	                }
	                else
	                {
	                    System.err.println ("Error : Could not get visual component");
	                }
				}
			}
			catch(Exception e)
			{
				System.out.println(e);
			}
		}
	}
 
    /* -------------------------------------------------------------------
     * grabs a frame's buffer from the web cam / device
     *
     * @returns A frames buffer
     * ------------------------------------------------------------------- */
 
    public Buffer grabFrameBuffer ( )
    {
        if ( player != null )
        {
            FrameGrabbingControl fgc = (FrameGrabbingControl)player.getControl ( "javax.media.control.FrameGrabbingControl" );
            if ( fgc != null )
            {
                return ( fgc.grabFrame() );
            }
            else
            {
                System.err.println ("Error : FrameGrabbingControl is null");
                return ( null );
            }
        }
        else
        {
            System.err.println ("Error : Player is null");
            return ( null );
        }
    }
 
 
    /* -------------------------------------------------------------------
     * grabs a frame's buffer, as an image, from the web cam / device
     *
     * @returns A frames buffer as an image
     * ------------------------------------------------------------------- */
 
    public Image grabFrameImage ( )
    {
        Buffer buffer = grabFrameBuffer();
        if ( buffer != null )
        {
            // Convert it to an image
            BufferToImage btoi = new BufferToImage ( (VideoFormat)buffer.getFormat() );
            if ( btoi != null )
            {
                Image image = btoi.createImage ( buffer );
                if ( image != null )
                {
                    return ( image );
                }
                else
                {
                    System.err.println ("Error : BufferToImage cannot convert buffer");
                    return ( null );
                }
            }
            else
            {
                System.err.println ("Error : cannot create BufferToImage instance");
                return ( null );
            }
        }
        else
        {
            System.out.println ("Error : Buffer grabbed is null");
            return ( null );
        }
    }
 
 
    /* -------------------------------------------------------------------
     * Closes and cleans up the player
     *
     * ------------------------------------------------------------------- */
 
    public void playerClose ( )
    {
        if ( player != null )
        {
            player.close();
            player.deallocate();
            player = null;
        }
    }
 
    public void windowClosing ( WindowEvent e )
    {
        playerClose();
        System.exit ( 1 );
    }
 
    public void componentResized ( ComponentEvent e )
    {
        Dimension dim = guiWindow.getSize();
        boolean mustResize = false;
 
        if ( dim.width < MIN_WIDTH )
        {
             dim.width = MIN_WIDTH;
             mustResize = true;
        }
 
        if ( dim.height < MIN_HEIGHT )
        {
            dim.height = MIN_HEIGHT;
            mustResize = true;
        }
 
        if ( mustResize )
             guiWindow.setSize ( dim );
    }
    
    public void componentHidden( ComponentEvent e)
    {
    	// do nothing
    }
    
    public void componentShown(ComponentEvent e)
    {
    	// do nothing
    }
    
	public void componentMoved(ComponentEvent e)
	{
	}
    
	public void internalFrameActivated(InternalFrameEvent e)
	{
	}
	
	public void internalFrameClosed(InternalFrameEvent e)
	{
		playerClose();
	}
	
	public void internalFrameClosing(InternalFrameEvent e)
	{
	}
	
	public void internalFrameDeactivated(InternalFrameEvent e)
	{
	}
	
	public void internalFrameDeiconified(InternalFrameEvent e)
	{
	}
	
	public void internalFrameIconified(InternalFrameEvent e)
	{
	}
	
	public void internalFrameOpened(InternalFrameEvent e)
	{
	}
 
 
    protected void finalize ( ) throws Throwable
    {
        playerClose();
        super.finalize();
    }
    
    
   	public BufferedImage createBufferedImage(Image image)
	{
   		if(image instanceof BufferedImage) {
      		return (BufferedImage)image;
   		}
 		BufferedImage bi = new BufferedImage(image.getWidth(null), image.getHeight(null),
  		BufferedImage.TYPE_INT_ARGB); // ARGB to support transparency if in original image
  		Graphics2D g = bi.createGraphics();
  		g.drawImage(image, 0, 0, null);
  		g.dispose(); // supposedly recommended for cleanup...
 
  		return bi;
	}

 
    class MyToolBarAction extends AbstractAction
    {
        public MyToolBarAction ( String name, String imagefile )
        {
            // Note : Use version this if you supply your own toolbar icons
            // super ( name, new ImageIcon ( imagefile ) );
 
            super ( name );
        }
 
        public void actionPerformed ( ActionEvent event )
        {
            toolbarHandler ( this );
        }
    };
}