import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.event.*;
import javax.swing.*;

public class ParticleSpawnFilter extends EffectFilter
{
	private FloatImage particleImages;
	private FloatImage particleMasks;
	private int numParticleImages;
	private int particleImageHeight;
	private transient ProbabilityMap pmap;
		
	public ParticleSpawnFilter()
	{
		filterInputs = new FilterLink[3];
		filterInputs[0] = null;
		filterInputs[1] = null;
		filterInputs[2] = null;
		dead = false;
		filterName = "Particle Spawn Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
		
		particleImages = null;
		particleMasks = null;
	}
	
	private void makePMap(FloatImage src)
	{
		double[][] tmp = new double[src.getWidth()][src.getHeight()];
		
		double[][] srca;
		
		for(int j = 0; j < src.getHeight(); ++j)
		{
			for(int i = 0; i < src.getWidth(); ++i)
			{	
				tmp[i][j] = 0.0;
			
				for(int k = 0; k < 3; ++k)
				{
					srca = src.getChannelData(k);
					tmp[i][j] += (srca[i][j] + 1.0); 
				}
			}
		}
		
		if(pmap == null)
			pmap = new ProbabilityMap(tmp);
		else
			pmap.buildProb(tmp);
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);

		JPanel foo = new JPanel();
		
		JButton particleImagesButton = new JButton("Capture Particle Images");
		particleImagesButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				// must be a button press
				doParticleImagesCapture();
			}
		});
		foo.add(particleImagesButton);
		
		guiWindow.add(foo);
		
		guiWindow.setVisible(true);
	}
	
	public void doParticleImagesCapture()
	{
		if(filterInputs[1] != null)
		{
			FloatImage temp = filterInputs[1].getOutput();
			if(temp != null)
				particleImages = temp.duplicate();
		}
		
		if(filterInputs[2] != null)
		{
			FloatImage temp = filterInputs[2].getOutput();
			if(temp != null)
				particleMasks = temp.duplicate();
		}
		
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		switch(inputNum)
		{
		case 0:
			return "Probability in";
		case 1:
			return "Images In";
		case 2:
			return "Masks In";
		default:
			return "input";
		}
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null)
			{
				if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());
				
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				// clear out internal buffer
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						ored[i][j] = -1.0;
						ogreen[i][j] = -1.0;
						oblue[i][j] = -1.0;
					}
				}
						
				
				makePMap(temp);
				
				int numDots = (int)(pmap.getCumulativeSum() / 3.0);
				
				Point2D.Double tmpPoint;
				int tmpx, tmpy;
				
				for(int i = 0; i < numDots; ++i)
				{
					tmpPoint = pmap.randPoint();
					
					tmpx = (int)tmpPoint.x;
					tmpy = (int)tmpPoint.y;
					
					if(tmpx >= 0 && tmpx < imgWidth && tmpy >= 0 && tmpy < imgHeight)
					{
						ored[tmpx][tmpy] = 1.0;
						ogreen[tmpx][tmpy] = 1.0;
						oblue[tmpx][tmpy] = 1.0;
					}
				}
			}
			
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}