import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

public class RobotDisplay extends JComponent implements MouseListener
{
	protected final static double HALF_WIDTH = 5.0;
	protected final static double HALF_HEIGHT = 5.0;
	
	protected final static int ROBOT_PRAD = 20;
	
	protected double robotX;
	protected double robotY;
	protected double robotTheta;
	protected double targetX;
	protected double targetY;
	protected int pheight;
	protected int pwidth;
	
	public RobotDisplay()
	{
		initialize();
	}
	
	public void initialize()
	{
		robotTheta = 0.0;
		robotX = 0.0;
		robotY = 0.0;
		targetX = 0.0;
		targetY = 0.0;
		
		addMouseListener(this);
		
		System.out.println("Initialized!");
	}
	
	protected double convertXToScreen(double x)
	{
		return ((x / HALF_WIDTH) * (pwidth / 2)) + (pwidth / 2);
	}
	
	protected double convertYToScreen(double y)
	{
		return pheight - (((y / HALF_HEIGHT) * (pheight / 2)) + (pheight / 2));
	}
	
	public void paint(Graphics g)
	{
		Graphics2D g2 = (Graphics2D)g;
		
		pwidth = getWidth();
		pheight = getHeight();
		
		int cx = (int)convertXToScreen(robotX); //(int)((robotX / HALF_WIDTH) * (pwidth / 2)) + (pwidth / 2);
		int cy = (int)convertYToScreen(robotY);//(int)((robotY / HALF_HEIGHT) * (pheight / 2)) + (pheight / 2);
		
		//System.out.println("CX: " + cx + ", CY: " + cy);
		
		// draw the robot itself...
		g2.drawOval(cx - ROBOT_PRAD, cy - ROBOT_PRAD, ROBOT_PRAD*2, ROBOT_PRAD*2);
		
		// draw the direction
		int offsetx = (int)(Math.cos(robotTheta) * ROBOT_PRAD);
		int offsety = -(int)(Math.sin(robotTheta) * ROBOT_PRAD);
		
		g2.drawLine(cx, cy, cx + offsetx, cy + offsety);
		
		// draw the target
		int stx = (int)convertXToScreen(targetX);
		int sty = (int)convertYToScreen(targetY);
		
		g2.drawLine(stx - ROBOT_PRAD, sty - ROBOT_PRAD, stx + ROBOT_PRAD, sty + ROBOT_PRAD);
		g2.drawLine(stx + ROBOT_PRAD, sty - ROBOT_PRAD, stx - ROBOT_PRAD, sty + ROBOT_PRAD);
	}
	
	public void setRobotPos(double x, double y, double theta)
	{
		robotX = x;
		robotY = y;
		robotTheta = theta;
	}
	
	public void setTargetPos(double x, double y)
	{
		targetX = x;
		targetY = y;
	}
	
	public double getTargetX()
	{
		return targetX;
	}
	
	public double getTargetY()
	{
		return targetY;
	}
	
	public void mousePressed(MouseEvent e)
	{
		System.out.println("Clicked " + e.getX() + ", " + e.getY());
		
		targetX = ((double)(e.getX() - (pwidth / 2)) / (pwidth / 2)) * HALF_WIDTH;
		targetY = -((double)(e.getY() - (pheight / 2)) / (pheight / 2)) * HALF_HEIGHT;
		
		repaint();
	}
	
	public void mouseReleased(MouseEvent e)
	{
	}
	
	public void mouseClicked(MouseEvent e)
	{
	}
	
	public void mouseEntered(MouseEvent e)
	{
	}
	
	public void mouseExited(MouseEvent e)
	{
	}
	
	public static void main(String[] args)
	{
		JFrame theFrame = new JFrame("Frame!");
		theFrame.setSize(300,300);
		
		RobotDisplay roboDisplay = new RobotDisplay();
		roboDisplay.setRobotPos(-2.5, -2.5, Math.PI / 4.0);
		roboDisplay.setTargetPos(2.5, 0.0);
		
		theFrame.getContentPane().add(roboDisplay);
		theFrame.setVisible(true);
	}
}