public class RoombaControl implements Runnable
{
	protected final static int COM_PORT = 10;
	protected static RoombaControl instance;
	
	protected SimpleSerial simpleSerial;
	protected int sleepTime;
	protected boolean areRunning;
	
	protected double s_powerLEDColor;
	protected double s_powerLEDIntensity;
	protected boolean s_advanceLEDOn;
	protected boolean s_playLEDOn;
	
	protected boolean s_direct_drive;
	protected int s_right_velocity;
	protected int s_left_velocity;
	protected int s_total_velocity;
	protected int s_turning_radius;
	
	protected int s_total_distance;
	protected int s_total_angle;
	
	protected boolean s_bump_left;
	protected boolean s_bump_right;
	
	protected boolean s_wall;
	
	protected boolean s_caster_drop;
	protected boolean s_right_drop;
	protected boolean s_left_drop;
	
	protected boolean s_cliff_front_left;
	protected boolean s_cliff_front_right;
	protected boolean s_cliff_left;
	protected boolean s_cliff_right;
	
	protected int[] s_digital_outs;
	
	public static RoombaControl getInstance()
	{
		if(instance == null)
			instance = new RoombaControl();
		
		return instance;
	}
	
	protected RoombaControl()
	{
		//simpleSerial = new SimpleSerialNative(COM_PORT, 57600, 8, SimpleSerial.ONESTOPBIT, SimpleSerial.NOPARITY);
		// Lets try this with the javax.comm package instead...
		simpleSerial = new SimpleSerialJava(COM_PORT, 57600, 8, SimpleSerial.ONESTOPBIT, SimpleSerial.NOPARITY);
		areRunning = true;
		sleepTime = 30;
		
		// Setup default values...
		s_powerLEDColor = 0.0;
		s_powerLEDIntensity = 0.0;
		s_advanceLEDOn = false;
		s_playLEDOn = false;
		
		s_total_distance = 0;
		s_total_angle = 0;
		
		s_digital_outs = new int[8];
		for(int i = 0; i < 8; ++i)
			s_digital_outs[i] = 0;
		
		try { Thread.sleep(1000); } catch (InterruptedException e) {}
		
		flushStream();
		startOI();
		setFullMode();
		
		Thread t = new Thread(this);
		t.start();
	}
	
	public boolean bumpLeft()
	{
		return s_bump_left;
	}
	
	public boolean bumpRight()
	{
		return s_bump_right;
	}
	
	public void resetDistance()
	{
		s_total_distance = 0;
	}
	
	public void resetAngle()
	{
		s_total_angle = 0;
	}
	
	public double distanceMoved()
	{
		return (double)(s_total_distance) / 10.0;
	}
	
	public int angleRotated()
	{
		return s_total_angle;
	}
	
	public double angleRotatedInRadians()
	{
		return (double)s_total_angle*Math.PI/180.0;
	}
	
	public void drive(double velocity, double radius)
	{
		s_total_velocity = (int)(velocity * 500.0);
		s_turning_radius = (int)(radius * 2000.0);
		
		byte vel_low = (byte)(s_total_velocity & 0xFF);
		byte vel_high = (byte)((s_total_velocity >> 8) & 0xFF);
		
		byte rad_low = (byte)(s_turning_radius & 0xFF);
		byte rad_high = (byte)((s_turning_radius >> 8) & 0xFF);
		
		byte[] bytes = {(byte)137, vel_high, vel_low, rad_high, rad_low};
		
		sendDirectBytes(bytes);
	}
	
	public void digitalOut(int outNum, int val)
	{
		if(s_digital_outs[outNum] != val)
		{
			s_digital_outs[outNum] = val;
			
			int result = 0;
			
			for(int i = 0; i < 8; ++i)
				result = result | (s_digital_outs[i] << i);
				
			int[] values = {147, result};
			System.out.println("Result: " + result);
			sendValues(values);
		}
	}
	
	public void directDrive(double rightVel, double leftVel)
	{
		s_right_velocity = (int)(rightVel * 500.0);
		s_left_velocity = (int)(leftVel * 500.0);
		
		byte rvel_low = (byte)(s_right_velocity & 0xFF);
		byte rvel_high = (byte)((s_right_velocity >> 8) & 0xFF);
		
		byte lvel_low = (byte)(s_left_velocity & 0xFF);
		byte lvel_high = (byte)((s_left_velocity >> 8) & 0xFF);
		
		byte[] bytes = {(byte)145, rvel_high, rvel_low, lvel_high, lvel_low};
		
		sendDirectBytes(bytes);
	}
	
	public void startOI()
	{
		int[] vals = {128};
		sendValues(vals);
	}
	
	public void setSafeMode()
	{
		int[] vals = {131};
		sendValues(vals);
	}
	
	public void setFullMode()
	{
		int[] vals = {132};
		sendValues(vals);
	}
	
	// Led control things...
	protected void setLEDs()
	{
		int ledstate = 0;
		if(s_advanceLEDOn) ledstate += 8;
		if(s_playLEDOn) ledstate += 2;
		
		int powerColor = ((int)(s_powerLEDColor * 255.0)) % 256;
		int powerIntensity = ((int)(s_powerLEDIntensity * 255.0)) % 256;
		
		int[] vals = {139, ledstate, powerColor, powerIntensity};
		
		sendValues(vals);
	}
	
	public void setPlayLEDState(boolean state)
	{
		s_playLEDOn = state;
		setLEDs();
	}
	
	public void setAdvanceLEDState(boolean state)
	{
		s_advanceLEDOn = state;
		setLEDs();
	}
	
	public void setPowerLEDState(double color, double intensity)
	{
		s_powerLEDColor = color;
		s_powerLEDIntensity = intensity;
		setLEDs();
	}
	
	public void setSensorPollInterval(int milliseconds)
	{
		sleepTime = milliseconds;
	}
	
	public void flushStream()
	{
		byte[] dummy = simpleSerial.readBytes();
	}
	
	public void stopControl()
	{
		areRunning = false;
	}
	
	public void run()
	{
		int[] packetZeroRequest = {142, 0};
		byte[] results;
		
		while(areRunning)
		{
			// Read in the results here
			sendValues(packetZeroRequest);
			// Sleep a bit to give it time to reply...
			try { Thread.sleep(20); } catch (InterruptedException e) {}
			results = simpleSerial.readBytes();
			if(results.length == 26)
				parsePacketZero(results);
			//else
			//	System.out.println("Bad results packet length: " + results.length);
			
			try { Thread.sleep(sleepTime); } catch (InterruptedException e) {}
		}
	}
	
	public void parsePacketOne(byte[] bytes)
	{
		if(bytes == null || bytes.length == 0)
			return;
			
		int offset = 0;
		offset = parseBumpsAndWheelDrops(bytes, offset);
		offset = parseWall(bytes, offset);
		offset = parseCliffs(bytes, offset);
			
		
	}
	
	public void parsePacketZero(byte[] bytes)
	{
		if(bytes == null || bytes.length == 0)
			return;
			
		int offset = 0;
		offset = parseBumpsAndWheelDrops(bytes, offset);
		offset = parseWall(bytes, offset);
		offset = parseCliffs(bytes, offset);
		// skip a bunch of things...
		offset += 6;
		offset = parseDistance(bytes, offset);
		offset = parseAngle(bytes, offset);
			
		
	}
	
	public int parseBumpsAndWheelDrops(byte[] bytes, int offset)
	{
		byte theByte = bytes[offset];
		
		s_caster_drop = (theByte & (0x01 << 4)) != 0;
		s_left_drop = (theByte & (0x01 << 3)) != 0;
		s_right_drop = (theByte & (0x01 << 2)) != 0;
		s_bump_left = (theByte & (0x01 << 1)) != 0;
		s_bump_right = (theByte & 0x01) != 0;
		
		return (offset + 1);
	}
	
	public int parseWall(byte[] bytes, int offset)
	{
		s_wall = bytes[offset] != 0;
		return (offset + 1);
	}
	
	public int parseDistance(byte[] bytes, int offset)
	{
		s_total_distance += (((int)bytes[offset]) << 8) | (int)(bytes[offset+1]);
		return offset+2;
	}
	
	public int parseAngle(byte[] bytes, int offset)
	{
		s_total_angle += (((int)bytes[offset]) << 8) | (int)(bytes[offset+1]);
		return offset+2;
	}
	
	public int parseCliffs(byte[] bytes, int offset)
	{
		s_cliff_left = bytes[offset] != 0;
		s_cliff_front_left = bytes[offset+1] != 0;
		s_cliff_front_right = bytes[offset+2] != 0;
		s_cliff_right = bytes[offset+3] != 0;
		
		return (offset + 4);
	}
	
	protected synchronized void sendDirectBytes(byte[] values)
	{
	    for(int i = 0; i < values.length; ++i)
	    {
			try { Thread.sleep(1); } catch (InterruptedException e) {}
	    	simpleSerial.writeByte(values[i]);
	    }
	}
	
	protected synchronized void sendValues(int[] values)
	{
	    for(int i = 0; i < values.length; ++i)
	    {
			try { Thread.sleep(1); } catch (InterruptedException e) {}
	    	simpleSerial.writeByte((byte)(values[i]));
	    }
	}
}