import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ScrollFilter extends EffectFilter
{	
	private transient JSlider xSlider;
	private transient JSlider ySlider;
	private double xPos;
	private double yPos;
	private double vx;
	private double vy;

	public ScrollFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Scroll Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		xPos = 0.0;
		yPos = 0.0;
		vx = 1.0;
		vy = 1.0;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		vx = ((double)xSlider.getValue()) / 100.0;
		vy = ((double)ySlider.getValue()) / 100.0;
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		JPanel whatever = new JPanel();
		
		xSlider = new JSlider(JSlider.HORIZONTAL);
		xSlider.setMajorTickSpacing(100);
		xSlider.setMinorTickSpacing(50);
		xSlider.setPaintTicks(true);
		xSlider.setBorder(BorderFactory.createTitledBorder("X Scroll Velocity"));
		xSlider.setMinimum(-1000);
		xSlider.setMaximum(1000);
		xSlider.setValue((int)(vx * 100.0));
		
		whatever.add(xSlider);
		
		ySlider = new JSlider(JSlider.HORIZONTAL);
		ySlider.setMajorTickSpacing(100);
		ySlider.setMinorTickSpacing(50);
		ySlider.setPaintTicks(true);
		ySlider.setBorder(BorderFactory.createTitledBorder("Y Scroll Velocity"));
		ySlider.setMinimum(-1000);
		ySlider.setMaximum(1000);
		ySlider.setValue((int)(vy * 100.0));
		
		whatever.add(ySlider);
		
		guiWindow.add(whatever);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "Input";
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		vx = ((double)xSlider.getValue()) / 100.0;
		vy = ((double)ySlider.getValue()) / 100.0;
		
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null)
			{
				if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
				{
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
				}
				
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				xPos += vx;
				yPos += vy;
				
				// This will fail if the image dimension is less than the speed
				if(xPos < 0.0)
					xPos += imgWidth;
				if(xPos >= imgWidth)
					xPos -= imgWidth;
					
				if(yPos < 0.0)
					yPos += imgHeight;
				if(yPos >= imgHeight)
					yPos -= imgHeight;
					
				int ipx = (int)xPos;
				int ipy = (int)yPos;
				
				int iy, ix;
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						ix = (i + ipx) % imgWidth;
						iy = (j + ipy) % imgHeight;
						
						ored[i][j] = ired[ix][iy];
						ogreen[i][j] = igreen[ix][iy];
						oblue[i][j] = iblue[ix][iy];
					}
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}