public class ServoControl implements Runnable
{
	protected final static int COM_PORT = 9;
	protected static ServoControl instance;
	
	protected SimpleSerial simpleSerial;
	protected int sleepTime;
	protected boolean areRunning;
	
	protected int[] servoPositions;
	protected boolean[] dirtyBits;
	
	public static ServoControl getInstance()
	{
		if(instance == null)
			instance = new ServoControl();
		
		return instance;
	}
	
	protected ServoControl()
	{
		//simpleSerial = new SimpleSerialNative(COM_PORT, 57600, 8, SimpleSerial.ONESTOPBIT, SimpleSerial.NOPARITY);
		// Lets try this with the javax.comm package instead...
		simpleSerial = new SimpleSerialJava(COM_PORT, 9600, 8, SimpleSerial.ONESTOPBIT, SimpleSerial.NOPARITY);
		areRunning = true;
		sleepTime = 100;
		
		// Setup default values...
		servoPositions = new int[8];
		dirtyBits = new boolean[8];
		
		for(int i = 0; i < 8; ++i)
		{
			servoPositions[i] = 128;
			dirtyBits[i] = false;
		}
		
		try { Thread.sleep(1000); } catch (InterruptedException e) {}
		
		Thread t = new Thread(this);
		t.start();
	}
	
	public void stopControl()
	{
		areRunning = false;
	}
	
	public synchronized void setServo(int servoNum, double pos)
	{
		if(pos < -1.0 || pos > 1.0 || servoNum < 1 || servoNum > 8)
			return;
		
		int intPos = ((int)((pos + 1.0) * 127.5)) % 256;
		
		// Make sure we aren't redundantly setting the position to where it
		//  already is...
		if(servoPositions[servoNum-1] != intPos)
		{
			servoPositions[servoNum-1] = intPos;
			dirtyBits[servoNum-1] = true;
		}
	}
	
	public void run()
	{
		while(areRunning)
		{
			// See which servos should be updated...
			for(int i = 0; i < 8; ++i)
			{
				if(dirtyBits[i])
				{
					String theString = "BD0SV" + (i+1) + "M" + servoPositions[i];
					//System.out.println(theString);
					sendString(theString);
					dirtyBits[i] = false;
					try { Thread.sleep(4); } catch (InterruptedException e) {}
				}
			}
			try { Thread.sleep(sleepTime); } catch (InterruptedException e) {}
		}
	}
	
	protected synchronized void sendString(String str)
	{
		byte[] values = str.getBytes();
	    
	    for(int i = 0; i < values.length; ++i)
	    {
			try { Thread.sleep(1); } catch (InterruptedException e) {}
			//System.out.println("Byte: " + values[i]);
	    	simpleSerial.writeByte(values[i]);
	    }
	    
	    simpleSerial.writeByte((byte)13);		
		
	}
	
	protected synchronized void sendValues(int[] values)
	{
	    for(int i = 0; i < values.length; ++i)
	    {
			try { Thread.sleep(1); } catch (InterruptedException e) {}
	    	simpleSerial.writeByte((byte)(values[i]));
	    }
	}
}