public class LightFollower implements GenericRobot
{
	protected final static int MODE_ROTATE = 1;
	protected final static int MODE_GO =2;
	protected final static int MODE_STOPPED = 3;
	
	protected final static double ROTATE_SPEED_FACTOR = 1.0;
	protected final static double MIN_ROTATE_SPEED = 0.1;
	protected final static double THETA_LIMIT = 0.1;
	protected final static double MAX_THETA_ERROR = 0.5;
	protected final static double MAX_SPEED = 3.0;
	protected final static double MIN_SPEED = 0.2;
	protected final static double GO_SPEED_FACTOR = 1.0;
	
	long lastTime;
	
	protected int mode;
	
	protected double targetX;
	protected double targetY;
	
	public void update(long dt, FilterLink[] inputs)
	{
		RoombaControl theControl = RoombaControl.getInstance();
		
		long curTime = System.currentTimeMillis();
		dt = curTime - lastTime;
		lastTime = curTime;
		
		if(dt < 0 || dt > 1000)
			dt = 20;
			
		// first, get the location and angle
		double curX = -1000.0;
		double curY = -1000.0;
		
		double curTheta = 0.0;
		
		if(inputs[0] != null && inputs[1] != null)
		{
			FloatImage p1i = inputs[0].getOutput();
			FloatImage p2i = inputs[1].getOutput();
			
			if(p1i != null && p2i != null)
			{
				double p1x, p2x, p1y, p2y;
				double[][] temput = p1i.getChannelData(0);
				p1x = temput[0][0];
				p1y = temput[1][0];
				
				temput = p2i.getChannelData(0);
				p2x = temput[0][0];
				p2y = temput[1][0];
				
				// we say the current position is the average of these two!
				curX = (p1x + p2x) / 2.0;
				curY = (p1y + p2y) / 2.0;
				
				// compute theta!
				curTheta = Math.atan2(p2y - p1y, p2x - p1x);
				if(curTheta < 0.0)
					curTheta += (Math.PI * 2.0);
			}
			else
			{
				return;
			}
		}
		else
		{
			return;
		}
		
		double thetaToTarget = Math.atan2(targetY - curY, targetX - curX);
		if(thetaToTarget < 0.0)
			thetaToTarget += (Math.PI * 2.0);
			
		
		if(mode == MODE_STOPPED)
		{
			// don't do anything!
		}
		else if(mode == MODE_GO)
		{
			if(Math.abs(targetTheta - curTheta) > MAX_THETA_ERROR)
			{
				mode = MODE_ROTATE;
				return;
			}
			
			double dx = targetX - curX;
			double dy = targetY - curY;
			double distance = Math.sqrt((dx*dx) + (dy*dy));
			
			if(distance < TARGET_DISTANCE)
			{
				mode = MODE_STOPPED;
				return;
			}
			
			double goSpeed = distance * GO_SPEED_FACTOR;
			goSpeed = Math.min(goSpeed, MAX_SPEED);
			goSpeed = Math.max(goSpeed, MIN_SPEED);
			
			theControl.directDrive(goSpeed, goSpeed);
		}
		else if(mode == MODE_ROTATE)
		{
			if(Math.abs(targetTheta - curTheta) < THETA_LIMIT)
			{
				mode = MODE_GO;
				return;
			}
			
			double diff = thetaToTarget - curTheta;
			double rSpeed = Math.abs(diff) * ROTATE_SPEED_FACTOR;
			rSpeed = Math.max(rSpeed, MIN_ROTATE_SPEED);
			rSpeed = Math.sign(diff) * rSpeed;
			
			theControl.directDrive(rSpeed, -rSpeed);
		}
	}
	
	public void init()
	{
		RoombaControl theControl = RoombaControl.getInstance();
		
		theControl.startOI();
		theControl.setSafeMode();
		
		mode = MODE_STOPPED;
	}
}