import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ThreeWaySwitchFilter extends EffectFilter
{	
	private transient JSlider switchSlider;
	private int switchPos;

	public ThreeWaySwitchFilter()
	{
		filterInputs = new FilterLink[3];
		filterInputs[0] = null;
		filterInputs[1] = null;
		filterInputs[2] = null;
		dead = false;
		filterName =  "Three-Way Switch" + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		
		switchPos = 0;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		switchPos = switchSlider.getValue();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		switchSlider = new JSlider(JSlider.HORIZONTAL);
		switchSlider.setMajorTickSpacing(1);
		switchSlider.setMinimum(0);
		switchSlider.setMaximum(2);
		switchSlider.setPaintTicks(true);
		switchSlider.setBorder(BorderFactory.createTitledBorder("Input Select"));
		switchSlider.setValue(switchPos);
		
		guiWindow.add(switchSlider);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "Input " + inputNum;
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		FloatImage sinput = null;
		
		switchPos = switchSlider.getValue();
		
		if(filterInputs[switchPos] != null)
		{
			sinput = filterInputs[switchPos].getOutput();
			
			if(sinput == null)
				return internalBuffer;
				
			return sinput;
		}
		else
		{
			return internalBuffer;
		}
	}	
}