import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ValueSliderFilter extends EffectFilter
{	
	private transient JSlider valueSlider;
	private double valuePos;

	public ValueSliderFilter()
	{
		filterInputs = new FilterLink[0];
		dead = false;
		filterName =  "Value Slider " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		
		valuePos = 0.0;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		valuePos = (valueSlider.getValue()) / 100.0;
	}
	
	public void rebuildFromSave()
	{
		// Make a 1x1, 1 channel image (nothing but a heavily-wrapped double)
		internalBuffer = new FloatImage(1, 1, 1);
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		valueSlider = new JSlider(JSlider.HORIZONTAL);
		valueSlider.setMajorTickSpacing(5);
		valueSlider.setMinimum(-100);
		valueSlider.setMaximum(100);
		valueSlider.setPaintTicks(true);
		valueSlider.setBorder(BorderFactory.createTitledBorder("Value"));
		valueSlider.setValue((int)(valuePos * 100.0));
		
		guiWindow.add(valueSlider);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "Input " + inputNum;
	}
	
	public String getOutputName(int outputNum)
	{
		return "DATA: value out";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		double doubleVal = valueSlider.getValue() / 100.0;
		
		internalBuffer.getChannelData(0)[0][0] = doubleVal;
		
		return internalBuffer;
	}	
}